//==================================================================
// Licensed Materials - Property of IBM
// (c) Copyright IBM Corp. 2003.  All rights reserved.
//
// US Government Users Restricted Rights - Use, duplication or
// disclosure restricted by GSA ADP Schedule Contract with IBM
// Corp.
//
// DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
// sample code created by IBM Corporation.  This sample code is
// not part of any standard or IBM product and is provided to you
// solely for the purpose of assisting you in the development of
// your applications.  The code is provided "AS IS", without
// warranty of any kind.  IBM shall not be liable for any damages
// arising out of your use of the sample code, even if they have
// been advised of the possibility of such damages.
//=================================================================

/**
 * @class public Tree
 * @constructor Tree
 * This class is tree control which will render the tree based on the
 *	data set which is getting from the tree adapter.
 * @param Object parent
 *	The object which is point to place where the datagrid will be put
 *	in the html page
 * @param String UrlPrefix
 *	The path relative to "jsl/datagrid/icons/" of the datagrid images
 * @param String xmldoc
 *	The xml file which has all the data. Since now we change to use
 *	emitter, this parameter will not be used.
 *
**/

function Tree (parent, UrlPrefix, xmldoc,type)
{
 //   this.IconLib = new TreeIconLibrary(UrlPrefix);
    this.UrlPrefix = UrlPrefix;
	this.treeIcons = [];
	
	//this.getDefaultSystemIcon(UrlPrefix);

	var xmlDoc;
    Log.debug("treecontrol.js.Tree()", "before LoadXMLString ") ;
	if (xmldoc==null)
    {
		if(type=="client")
			xmlDoc	= LoadXMLString("<startNode><root></root></startNode>");
		else
			xmlDoc	= LoadXMLString("<root />");

		//xmlDoc	= LoadXMLString("<?xml version=\"1.0\" encoding=\"utf-8\" ?><root />");
       	Log.debug("treecontrol.js.Tree()", "after LoadXMLString ") ;
    }
    else
		xmlDoc = LoadXML(xmldoc);

	if (null == xmlDoc)
		return this;

	// Properties
	this.rootItem = this;
	this.xmlNode = xmlDoc.documentElement;
	this.itemName = this.xmlNode.nodeName;
	this.crumtree = false;
	this.parentHTMLElement = parent;
	this.HTMLElement = null;
	
	//Array that will contain the treeItems representing each tree node
	this.a_index = [];
	//Array that will contain the Image objects for all images on the tree
	this.treeIconObjs=[];
	this.checkedItemArray = [];
	this.checkedEObjectArray = []; //keep all the selected eobjects for event object.
	
	this.childItems = [];
	this.eventArray = [];

	var nIndex = 0;
	for (var i = 0; i < xmlDoc.documentElement.childNodes.length; i++)
	{
		var node = xmlDoc.documentElement.childNodes[i];
		if (node.nodeType != 1)
			continue;

		new TreeItem(this, node, nIndex);
		nIndex++;
	}

	this.itemID = odcTrees.length;
	odcTrees[this.itemID] = this;

	this.styleIcons = [];

	// Size related stuff
	this.width = null;		// width of the tree
	this.height = null;		// height of the tree
	this.widthS = null;		// width of the tree with appropriate suffixes
	this.heightS = null;	// height of the tree with appropriate suffixes
	this.borderW = null;	// padding for margins/borders/etc. added to column widths by browser

}

	Tree.prototype.Adapter = null;
	Tree.prototype.parentItem = null;
	Tree.prototype.selectedItem = null;
	Tree.prototype.depth = -1;
	Tree.prototype.enableSelect = false;
	Tree.prototype.enableDrop = false;
	Tree.prototype.rootVisibleFlag = true;
	Tree.prototype.treeTableFlag = false;
	Tree.prototype.workPlaceFlag = false;
	Tree.prototype.dynamicFlag = false;
	Tree.prototype.userHighlight = false;
	Tree.prototype.treeTableArr = [];
	Tree.prototype.treeTableAttrs = [];
	Tree.prototype.iconLeafArray = null;
	Tree.prototype.height = null;
	Tree.prototype.width = null;
	Tree.prototype.styleClass = "";			// so two odcTrees can have different style sheet.
	Tree.prototype.styleOverflow = null;	// override how overflow on the outerdiv is handled
	Tree.prototype.alength = 0; //the tree's node length;
	Tree.prototype.type = "Tree";
	Tree.prototype.pageId = "tree1";//default pageId which will be changed by ODCRegistry when do addElementInfo


	// Methods


/**
 * @method private get_E
 * 	This is an utility function which is is used to get the html object based on the tag id
 * @param String s_id
 *	html tag id
**/
	Tree.prototype.getE = (document.all) ?	
		function (s_id) { return document.all[s_id] } :
		function (s_id) { return document.getElementById(s_id) };

/**
* @method public Tree.prototype.getType
*   This function is return the type of the control
* @return String
*	The type of the control, "Tree".
**/
	Tree.prototype.getType = function(){
		return this.type;
	}


/**
* @method public Tree.prototype.getId
*   This function return the id of the control
* @return String
* 	return the pageID of this control.
**/
	Tree.prototype.getId = function(){
		return this.pageId;
	}


/**
* @method public Tree.prototype.setStyleClass
*   This function is used to set the style sheet prefix for this tree
* @param String prefix
*	the prefix which is added before the style sheet name. Like "tree1",
*	so the css name will be "tree1_labelNormalStyle".
**/
 Tree.prototype.setStyleClass = function(prefix)
 {
	 this.styleClass = prefix + "_";
	 this.parentHTMLElement.className = this.styleClass+"classForTreeSysIcons";
	 var icons = new iconLibrary(this.parentHTMLElement,"list-style-image");
	 for(var key in icons){
		 this.treeIcons[key] = icons[key];
	 }
 }

/**
* @method public Tree.prototype.setStyleOverflow
*   This function is used to override the "overflow" management on the outer div
* @param String value
*	Value to set the overflow to
**/
 Tree.prototype.setStyleOverflow = function(value)
 {
	 this.styleOverflow = value;
 }

/**
* @method public Tree.prototype.loadImages
*   This function creates image objects and add the source to them
*   to force preloading of images to ensure all images appear on first
*   rendering of the page.
**/
Tree.prototype.loadImages =function()
{
	for(key in this.treeIcons)
	{
		var imgObj= new Image();
		imgObj.src=this.treeIcons[key];
		this.treeIconObjs[key]=imgObj;
	}
}
 


/**
* @method public Tree.prototype.addEvent
*   This function is used to add event handler to the control array so
*	that all the event can be queued
* @param String action
*	The action which the datagrid control has event handler associated,
*	like "OnHighlight", "OnSelect" etc.
* @param String handler
*   The name of the function which wrap all the processings.
* @param String eclassType
*	to have the same API cross all the controls for select&active,
*	select&set event emitter, need to have this parameter to set the
*	node level handler
*
**/
	Tree.prototype.addHandler = function(action, handler,eclassType)
	{
		uaction=action.toUpperCase();
		Log.debug("TreeControl.js addHandler(): ","uaction is "+uaction);
		//If action is onnodedrop set global enableDrop flag to true
		if(uaction=="ONNODEDROP")
		{
			Tree.prototype.enableDrop = true;
			Log.debug("TreeControl.js addHandler(): ","enableDrop is true");
		}
	
		if(eclassType!=null&&eclassType!='undefined'&&eclassType!="")
		{
			//to have the same API cross all the controls for select&active, select&set event emitter, do this extra step.
			var str = eclassType + ":" + action + ","+handler;
			this.Adapter.addNodeEventHandler(str);
		}else{
			this.eventArray[this.eventArray.length] = new Array(action, handler);
		}


	}

/**
* @method public Tree.prototype.addBehavior
*   This function is used to add event handler to the control array so
*	that all the events can be queued
* @param String event
*	The event e.g. onnodecollapse, onnodeexpand, onnodehighlight
* @param String func
*   The name of the function which is passed in via the onactionfunction tag. This allows user defined events to be triggered
* @param String actionObjectr
*  The genericBehavior object if using the behaviorAction and targetAction tags. (e.g alert, show, hide)
* @param String eclassType
*	to have the same API cross all the controls for select&active,
*	select&set event emitter, need to have this parameter to set the
*	node level handler
*
**/
	Tree.prototype.addBehavior = function(event, func , actionObject)
	{
			//Can have both a generic behavior and a user customized one attached at same time
			e=event.toUpperCase();
			
			//If event is onnodedrop set global enableDrop flag to true
			if(e=="ONNODEDROP")
				Tree.prototype.enableDrop = true;

			this.eventArray[this.eventArray.length] = new Array(event, func, actionObject);
			
	
	}

/**
* @method public Tree.prototype.addGenericBehavior
*   This function is used to handle "dropping".
**/
	Tree.prototype.addGenericBehavior= function(action, object)
	{
		uaction=action.toUpperCase();
		Log.debug("TreeControl.js addGenericBehavior(): ","uaction is "+uaction);
		
		//If action is onnodedrop set global enableDrop flag to true
		if(uaction=="ONNODEDROP")
		{
			Tree.prototype.enableDrop = true;
			Log.debug("TreeControl.js addGenericBehavior(): ","enableDrop is true");
		}
	


	}

/**
* @method public Tree.prototype.reSize
*   This function is used to resize tree when window is resized.
**/
	Tree.prototype.reSize = function()
	{
		// Currently a no-op
		// We size all the items using either "px" (so they are invariant when the window sizes)
		// Or as "%" (and they will resize as the parent div resizes)
	}

/**
* @method public Tree.prototype.setHeight
*   This function is used to set the tree's height.
* @param string height
*	The height of the tree
**/

	Tree.prototype.setHeight = function(height)
	{
		// We expect either no suffix, "px" or "%".
		this.height = height;
		if (this.hasSizeSuffix(height)) {
			this.heightS = height;
			if (height.indexOf("px")>=0) this.height=height.substr(0,height.length-2);
		} else this.heightS = height+"px";
	}

/**
* @method public Tree.prototype.setWidth
*   This function is used to set the tree's width.
* @param string width
*	The width of the tree
**/

	Tree.prototype.setWidth=function(width)
	{
		// We expect either no suffix, "px"  or "%".
		this.width = width;
		if (this.width.length=0) 
			this.widthS = this.width;
		else if (this.hasSizeSuffix(width)) {
			this.widthS = width;
			if (width.indexOf("px")>=0) this.width=width.substr(0,width.length-2);
		} else this.widthS = width+"px";
		if (this.treeTableFlag) this.adjustWidth();
	}

/**
* @method public Tree.prototype.adjustWidth
*   This function is used to set the ajdust a tree table's width to be at least the sum of the column widths
* @param string width
*	The width of the tree
**/
	Tree.prototype.adjustWidth=function(width) {
		// I't questionable if we should do this, but if the sum of the column widths (assuming fixed width columns)
		// in a tree table is.  This is difficult to do as FF and IE differ as to whether the width you specify for
		// a column includes borders/margins or not.  So what we do is we 
		if (this.treeTableFlag && this.treeTableArr!=null && this.widthS!=null && this.widthS.length>0 && this.widthS.substr(this.widthS.length-2)=="px") {
			var i, w, min=0;
			for(i=this.treeTableArr.length-1;i>=0;i--) {
				w = this.getColWidth(i);
				if (w.W!=null && w.isPX) min += parseInt(w.W,10);
			}
			min += (this.borderW!=null)?this.borderW:0;
			if (isIE()) min +=2;
			if (min>parseInt(this.width,10)) {
				this.width = min+"";
				this.widthS = min+"px";
				if (this.HTMLElement!=null) this.HTMLElement.style.width = this.widthS;
			}
		}
	}

/**
 * @method private Tree.prototype.hasSizeSuffix
 * 	This method is used to parse width from a string to see if it has a "size" suffix such as "%" or "px"
**/
	Tree.prototype.hasSizeSuffix = function(v) {
		var r=false, digits="0123456789";
		if (v!=null&&v.length>0) r = (digits.indexOf(v.charAt(v.length-1))<0);
		return r;
	}

/**
 * @method private Tree.prototype.getColWidth
 * 	This method is used to parse width from a string 
**/
	Tree.prototype.getColWidth = function(col) {
		var w="", wS="", PX=false, digits="0123456789";
		if(this.treeTableAttrs!=null && this.treeTableAttrs.length>0) {
			w = wS = this.treeTableAttrs[col];
			if (wS!=null && wS.length > 0 && wS.toLowerCase()!="null") {
				if(wS.indexOf("px")>=0)
					w = wS.substring(0,wS.indexOf("px"));
				else if (!this.hasSizeSuffix(wS))
					wS = w + "px";
				PX = (wS.substr(wS.length-2)=="px");
			} else w = wS = "";
		}
		return {W:w, WS:wS, isPX:PX};
	}

/**
* @method public Tree.prototype.addTreeTable
*   This function is used to activate the treeTable implementation if a treeTable is emitted to the page
* @param string id
*	id of treeTable component
**/
Tree.prototype.addTreeTable=function(id)
{
	this.treeTableFlag="true";
	this.treeTableId=id;
}

/**
* @method public Tree.prototype.setColumnAttributes
*   This function is used to set the widths and headers passed in via the tableColumn tag
* @param string with string	
*	String containing col/width pairs in the following format: colLabel,colWidth; colLabel,colWidth;
**/
	Tree.prototype.setupColumnAttributes=function(attrString)
	{
		//Separate the information into an array of labels and widths
		var i, index, pairArray = attrString.split(";");
		
		for(i=0;i<pairArray.length;i++)
		{
			index=pairArray[i].indexOf(",");
			if(pairArray[i].substring(0,index)!="")
				this.treeTableArr[i]=pairArray[i].substring(0,index);
			this.treeTableAttrs[i]=pairArray[i].substring(index+1,pairArray[i].length);
		}	
	}

/**
 * @method public findItemByAnchor
 * Returns the TreeItem associated with the specified HTML anchor element, 
 * or null if the specified element does not belong to a tree node
 * @param anchorElem
 *	 HTML anchor element reference for a tree node
 **/
Tree.prototype.findItemByAnchor = function(anchorElem) 
{
	var item = null;
	if (anchorElem.id.indexOf("i_a") == 0) {
		var id = anchorElem.id.substring(3).split('_');
		var ctrl = odcTrees[id[0]];
		if (ctrl) 
		{
			item = ctrl.a_index[id[1]];
		}
	}
	return item;
}

/**
 * @method public findItemByEObject
 * Returns the TreeItem associated with the specified eobject
 * @param eobject
 *	 The eobject belonging to the tree item
 **/
Tree.prototype.findItemByEObject = function(eobject)
{
	var item = null;
	if (eobject) {
		for(var j=0;j<this.a_index.length;j++)
		{	
			//find the treeItem corresponding to the eobject
			if(this.a_index[j].eobj.ID==eobject.ID)
			{
				item = this.a_index[j];
				break;
			}
		}
	}
	return item;
}

/**
 * @method private Tree.prototype.writeTreeUI1
 * 	This code creates the UI of the tree. As it writes directly to the document object,
 *	it only runs once when the tree is initially constructed.
**/

	Tree.prototype.writeTreeUI1 = function() {

		if (this.parentHTMLElement==null)
			return; // throw error
		
		//if there are nodes already added remove them
		var i;
		if(this.parentHTMLElement.innerHTML!="") {
			for(i = (this.parentHTMLElement.childNodes.length - 1) ; i >= 0; i--)
			{	// plus you have to remove in reverse order 
				if(this.parentHTMLElement.childNodes[i] != null)
					this.parentHTMLElement.removeChild(this.parentHTMLElement.childNodes[i]);
			}
		}
		
		//Create images objects to force images to load on first render
		this.loadImages();
		
		var child, hdr, div=document.createElement("div");		
		div.className = this.styleClass+"outerTreeStyle";
		div.setAttribute("id","i_div" + this.itemID + "_rootdiv");
		div.style.overflow = (this.styleOverflow!=null)?this.styleOverflow:"hidden";

		//If height is explicitly specified then set that height and enable scrolling in case tree exceeds that height
		if(this.height!=null&&this.heightS.length>0) {
			div.style.height = this.heightS;
			if (this.styleOverflow==null) div.style.overflowY = "auto";
		}

		// If width is explicitly specified, then set the width and enable scrolling if needed
		// NOTE: if this is a treetable, we'll later adjust the width to equal the total of the column widths if this
		// is needed
		if(this.width!=null&&this.widthS.length>0) {
			div.style.width = this.widthS;
			if (this.styleOverflow==null) div.style.overflowX = "auto";	
		}

		//If it is a tree table add the headers for the columns
		if(this.treeTableFlag) {
		 	hdr=this.createHeader();
		 	div.appendChild(hdr);
		}
		
		for (i = 0; i < this.childItems.length; i++) {				
			child = this.childItems[i].init();
			div.appendChild(child[0]);
			if(child[1] != null) div.appendChild(child[1]);
			this.childItems[i].open();	
		}

		this.HTMLElement = div;
		this.parentHTMLElement.appendChild(div);

		// Try to fix up any width errors in the tree table (making the table bigger)
		if(this.treeTableFlag) {
			var f=false,h = this.HTMLElement.firstChild;
			if (this.borderW==null && h.rows && h.rows.length>0) {
				this.borderW = 0;
				h = h.rows[0];
				for (i=0;!f && i<h.cells.length;i++) {
					if (!this.hasSizeSuffix(h.cells[i])) {
						f = true;
						this.borderW =(h.cells[i].offsetWidth-h.cells[i].width);
						if (this.borderW<0) this.borderW=0;
						this.borderW = this.borderW*h.cells.length;					
					}
				}
			}
			this.adjustWidth();
		}
	}
	
/**
 * @method private Tree.prototype.createHeaderCell
 * 	Create surrounding table with header and footer styles for treeTable
**/
	Tree.prototype.createHeader  = function ()
	{	
		//Set style classes
		var headerStyle= this.styleClass+"headerStyle", nodeStyle= this.styleClass+"nodeStyle", nodeLableStyle=this.styleClass+"labelNormalStyle";
		
		//Create table and cells for this header "row" of the Tree Table. (each row is a seperate table)
		var t = document.createElement("table");
		t.setAttribute("id","i_headerTab_" + this.itemID + "_" + this.itemID);
		t.cellPadding = t.border = t.cellSpacing = "0";
		t.className=nodeStyle;
		var i, td, lbl, newW, tr = t.insertRow(0);
		tr.setAttribute("id","i_headTR_" + this.itemID + "_" + this.itemID);

		for(i=this.treeTableArr.length-1;i>=0;i--) {
			td = tr.insertCell(0);
			td.setAttribute("id","i_TD_" +i+"_"+ this.itemID + "_" + this.itemID);
			td.className=headerStyle;

			lbl = document.createElement("label");
			lbl.setAttribute("id","i_headTxt_"+i+"_"+ this.itemID + "_" + this.itemID);
			lbl.className = nodeLableStyle;
			lbl.appendChild(document.createTextNode(" "+this.treeTableArr[i]));

			// Calc the width of this element (if one is given)
			newW = this.getColWidth(i);
			if (newW!=null&&newW.W!=null) {
				// Set the width on BOTH the td and the label inside (so the contents of the cell will wrap)
				td.width=newW.W;
				td.style.width=newW.WS;
				lbl.style.display="block";
				lbl.style.width=newW.WS;
			}
			td.appendChild(lbl);
		}
		return t;
	}

/**
 * @method public Tree.prototype.show
 * 	This method is used to expand the root node to one level deep
 *	(because of performance fix) at the first time right now. Later
 *	we will fix the function openfoldr(level) to allow user open
 *	any level of node the first time the tree is displayed.
**/
	Tree.prototype.show = function()
	{
		this.updateControl();
	}


/**
 * @method private Tree.prototype.updateControl
 * 	This method is used to expand the root node to one level deep
 *	(because of performance fix) at the first time right now. Later
 *	we will fix the function openfoldr(level) to allow user open
 *	any level of node the first time the tree is displayed.
**/
	Tree.prototype.updateControl = function()
	{
		if(this.a_index==null||this.a_index.length==0){
			alert(action_obfcontrolrender_errormsg);
			return;
		}

		this.rootItem.expand(0);

	}

/**
 * @method private Tree.prototype.openfolder
 * 	This method is used to expand the root node to any level deep based on the user's setting.
 *  Now this method doesn't work because of fix in performance issue.
 * @param int itemID
 *	The deep level the user want to open.
**/

	Tree.prototype.openfolder = function (itemID)
	{
		if(isNaN(itemID) || (itemID> this.a_index.length))
			return;

		if(itemID==-1) this.rootItem.expandAllInternal(this.rootItem.itemID);
		if(itemID==0) this.a_index[0].open(this.a_index[0].isOpened);

		for (var j = 1; j < itemID; j++)
		{
			var itemObj = this.a_index[j];
			itemObj.open(itemObj.isOpened)
		}
	}


/**
 * @method private initTree
 * 	This method is not used right now
 * @param String url
 *	The url of the data model xml file
**/

	Tree.prototype.initTree = function(url)
	{
		var htmlElement = this.getE('i_div' + this.rootItem.itemID + '_' + this.rootItem.itemID);
		if (!htmlElement)
		{
			htmlElement = this.getE('i_txt' + this.rootItem.itemID + '_' + this.rootItem.itemID);
			if (!htmlElement)
				return;
		}

		var xmlDoc = LoadXML(url);
		if (null == xmlDoc)
			return;

		this.xmlNode = xmlDoc.documentElement;
		this.itemName = this.xmlNode.nodeName;

		this.a_index = null;
		this.childItems = null;
		this.a_index = [];
		this.childItems = [];


		var nIndex = 0;
		for (i = 0; i < this.xmlNode.childNodes.length; i++)
		{
			var node = this.xmlNode.childNodes[i];
			if (node.nodeType != 1)
				continue;

			new TreeItem(this, node, nIndex);
			nIndex++;
		}

		/*htmlElement.parentNode.innerHTML = this.childItems[0].init();*/
		var tempElem = this.childItems[0].init();
		htmlElement.parentNode.appendChild(tempElem[0]);
		if(tempElem[1] != null)
			htmlElement.parentNode.appendChild(tempElem[1]);
		this.childItems[0].open();
	};

	Tree.prototype.loadLeaf = function(itemID, url)
	{
		var treeItem = this.a_index[itemID];
		if (treeItem == null)
			return;

		var htmlElement = this.getE('i_div' + this.rootItem.itemID + '_' + itemID);
		if (!htmlElement)
		{
			htmlElement = this.getE('i_txt' + this.rootItem.itemID + '_' + itemID);
			if (!htmlElement)
				return;
		}

		var xmlDoc = LoadXML(url);
		if (null == xmlDoc)
			return;

		var xmlNode = xmlDoc.documentElement;
		var nIndex = 0;
		for (i = 0; i < xmlNode.childNodes.length; i++)
		{
			var node = xmlNode.childNodes[i];
			if (node.nodeType != 1)
				continue;

			new TreeItem(treeItem, node, nIndex);
			nIndex++;
		}

		/*htmlElement.parentNode.innerHTML = treeItem.init();*/
		var tempElem = treeItem.init();
		htmlElement.parentNode.appendChild(tempElem[0]);
		if(tempElem[1] != null)
			htmlElement.parentNode.appendChild(tempElem[1]);
		treeItem.open();
	};
/**
 * @method private addItem
 * 	This method is not used right now since the current tree is not editable.
**/
	Tree.prototype.addItem = function(itemID, displayText, value)
	{
		if (displayText == null || displayText == "")
			return;
		var futureParentItemObj = this.a_index[itemID];
		if (futureParentItemObj == null)
			return;

		var wasOpened = futureParentItemObj.isOpened;

		var htmlElement = this.getE('i_div' + this.rootItem.itemID + '_' + itemID);
		if (!htmlElement)
		{
			htmlElement = this.getE('i_txt' + this.rootItem.itemID + '_' + itemID);
			if (!htmlElement)
				return;
		}

		var htmlRootElement = this.getE('i_div' + this.rootItem.itemID + '_' + this.rootItem.itemID);
		if (!htmlRootElement)
		{
			htmlRootElement = this.getE('i_txt' + this.rootItem.itemID + '_' + this.rootItem.itemID);
			if (!htmlRootElement)
				return;
		}

		var newXmlNode = futureParentItemObj.xmlNode.ownerDocument.createElement(displayText);
		var textValueNode = futureParentItemObj.xmlNode.ownerDocument.createTextNode(value);
		newXmlNode.appendChild(textValueNode);

		futureParentItemObj.xmlNode.appendChild(newXmlNode);

		var newItem = new TreeItem(futureParentItemObj, newXmlNode, futureParentItemObj.childItems.length);

		if (this.depth <= 1)
		{
			/*htmlRootElement.parentNode.innerHTML = futureParentItemObj.rootItem.childItems[0].init();*/
			var tempElem = futureParentItemObj.rootItem.childItems[0].init();			
			htmlRootElement.parentNode.appendChild(tempElem[0]);
			if(tempElem[1] != null)
				htmlRootElement.parentNode.appendChild(tempElem[1]);
			futureParentItemObj.rootItem.childItems[0].open();
		}
		else
		{
			/*htmlElement.parentNode.innerHTML = futureParentItemObj.init();*/
			var tempElem = futureParentItemObj.init();			
			htmlElement.parentNode.appendChild(tempElem[0]);
			if(tempElem[1] != null)
				htmlElement.parentNode.appendChild(tempElem[1]);
		}	

		//htmlRootElement.parentNode.innerHTML = futureParentItemObj.rootItem.childItems[0].init();

		//futureParentItemObj.rootItem.childItems[0].open();
		futureParentItemObj.open(!wasOpened);

		return newItem;
	}


/**
 * @method private addItem
 * @param Object item
 *	 Parent node which the new node will be attached on.
 * @param String displayText
 *	 The text value of the new added node.
**/
	Tree.prototype.addNode = function(item, displayText)
	{
		var itemID = item.itemID;

		if (displayText == null || displayText == "")
			return;
		var futureParentItemObj = this.a_index[itemID];
		if (futureParentItemObj == null)
			return;

		var wasOpened = futureParentItemObj.isOpened;

		var newXmlNode = futureParentItemObj.xmlNode.ownerDocument.createElement(displayText);

		futureParentItemObj.xmlNode.appendChild(newXmlNode);

		var newItem = new TreeItem(futureParentItemObj, newXmlNode, futureParentItemObj.childItems.length);


		return newItem;
	}

/**
 * @method private updateTreeItem
 * 	This method is not used right now since the current tree is not editable.
**/
	function updateTreeItem(itemObj, editElem, htmlElement, isUpdate)
	{
		newXMLElem = itemObj.xmlNode.ownerDocument.createElement(editElem.value);
		newXMLTextNode = itemObj.xmlNode.ownerDocument.createTextNode(getXMLNodeValue(itemObj.xmlNode));
		newXMLElem.appendChild(newXMLTextNode);
		var retObj = itemObj.xmlNode.parentNode.replaceChild(newXMLElem, itemObj.xmlNode);
		var strOldName = itemObj.itemName;
		itemObj.xmlNode = newXMLElem;
		itemObj.parentItem.xmlNode = itemObj.xmlNode.parentNode;
		itemObj.itemName = editElem.value;
		itemObj.rootItem.a_index[itemObj.itemID] = itemObj;

		strInnerHTML = htmlElement.innerHTML;
		strInnerHTML = strInnerHTML.substring(0, strInnerHTML.length - strOldName.length) + editElem.value;
		htmlElement.innerHTML = strInnerHTML;
		editElem.parentNode.replaceChild(htmlElement, editElem);
	}

/**
 * @method private editItem
 * 	This method is not used right now since the current tree is not editable.
**/
	Tree.prototype.editItem = function(itemID)
	{
		var itemObj = this.a_index[itemID];
		if (itemObj == null)
			return;

		var htmlElement = this.getE('i_div' + this.rootItem.itemID + '_' + itemID);
		if (!htmlElement)
		{
			htmlElement = this.getE('i_txt' + this.rootItem.itemID + '_' + itemID);
			if (!htmlElement)
				return;
		}
		var strEditControlContent = "<INPUT id='txtItem' type='text' size='9' name='txtItem'>";
		var editElem = null, textNode = null;
		if (navigator.appName == "Netscape")
		{
			editElem = htmlElement.ownerDocument.createElement("input");
			textNode = htmlElement.ownerDocument.createTextNode(itemObj.itemName);
			editElem.setAttribute("type", "text");
			editElem.setAttribute("size", "9");
			editElem.style.height = "17px";
			editElem.style.fontSize = 10;
			//editElem.appendChild(textNode);
		}
		else
		{
			editElem = htmlElement.ownerDocument.createElement(strEditControlContent);
			editElem.style.height = "17px";
			editElem.style.fontSize = 10;
		}

		htmlElement = htmlElement.parentNode.replaceChild(editElem, htmlElement);

		editElem.value = itemObj.itemName;
		editElem.focus();
		editElem.select();

		if (this.onstartedit != null)
			this.onstartedit(itemObj, editElem);

		editElem.onblur = function onBlurEditBox()
		{
			if (this.onfinishedit != null)
			{
				var isUpdate = false;
				isUpdate = this.onfinishedit(itemObj, editElem);
				this.updateTreeItem(itemObj, editElem, htmlElement, isUpdate);
			}
			else
				this.updateTreeItem(itemObj, editElem, htmlElement, true);
		}
	}

/**
 * @method private removeItem
 * 	This method is not used right now since the current tree is not editable.
**/
	Tree.prototype.removeItem = function(itemID)
	{
		var itemObj = this.a_index[itemID];
		if (itemObj == null)
			return;

		var isLeaf = false;
		var htmlNodeName = 'i_div' + this.rootItem.itemID + '_' + itemID;
		var htmlElement = this.getE(htmlNodeName);
		if (!htmlElement)
		{
			htmlNodeName = 'i_txt' + this.rootItem.itemID + '_' + itemID;
			htmlElement = this.getE(htmlNodeName);
			if (!htmlElement)
				return;
			isLeaf = true;
		}

		if (navigator.appName == "Netscape")
		{
			var parent = htmlElement.parentNode;
			if (isLeaf)
			{
				htmlElement.parentNode.removeChild(htmlElement);
				parent.parentNode.removeChild(parent);
			}
			else
			{
				htmlElement.parentNode.removeChild(htmlElement);
				htmlNodeName = 'i_txt' + this.rootItem.itemID + '_' + itemID;
				var htmlElement = this.getE(htmlNodeName);
				parent = htmlElement.parentNode;
				htmlElement.parentNode.removeChild(htmlElement);
				parent.parentNode.removeChild(parent);
			}
		}
		else
		{
			if (isLeaf)
				htmlElement.parentElement.removeNode(true);
			else
			{
				htmlElement.removeNode(true);
				htmlNodeName = 'i_txt' + this.rootItem.itemID + '_' + itemID;
				var htmlElement = this.getE(htmlNodeName);
				htmlElement.parentElement.removeNode(true);
			}
		}

		itemObj.xmlNode.parentNode.removeChild(itemObj.xmlNode);
		var indexInParent = 0;
		// Find index of the removable item in the parent childs array
		for (i = 0; i < itemObj.parentItem.childItems.length; i++)
		{
			if (itemObj.parentItem.childItems[i] == itemObj)
			{
				indexInParent = i;
				break;
			}
		}
		itemObj.parentItem.childItems.splice(indexInParent, 1);
		itemObj.childItems.splice(0, itemObj.childItems.length);
		//this.a_index.splice(itemID, 1);
		itemObj = null;
	}

/**
 * @method private Tree.prototype.select
 * 	This method is used to highlight the node which is be selected
 * @param int itemID
 *	The position of the node in the whole big array which includes all the rendered
 *	nodes.
**/
	Tree.prototype.select = function (itemID,nativeEvt)
	{
		return this.a_index[itemID].select(null,null,nativeEvt);
	}

/**
 * @method private Tree.prototype.check
 * 	This method is used to call onselect event if onselect is defined when the node's
 *  checkbox is checked or unchecked.
 * @param int itemID
 *	The position of the node in the whole big array which includes all the rendered
 *	nodes.
**/

	Tree.prototype.check = function(selectObj, itemID, nativeEvt)
	{

		if(selectObj.checked)
		{
			return this.a_index[itemID].check('ONSELECT',null,nativeEvt);

		}else
		{
			return this.a_index[itemID].check('ONUNSELECT',null,nativeEvt);
		}

	}

/**
 * @method private Tree.prototype.drop
 * 	This method is used to call drop event
 * @param int itemID
 *	The position of the node in the whole big array which includes all the rendered
 *	nodes.
 * @ param int nativeEvent
 *	Native JavaScript event object
 *
**/

	Tree.prototype.drop = function(itemID, nativeEvent)
	{
		var itemObj = this.a_index[itemID];
	
		if (itemObj == null)
			return;
		
		if(!itemObj.activeTreeEvent("onNodeDrop", nativeEvent))
			return false;

		if(this.a_index[itemID])
			this.a_index[itemID].upstatus(true,"dropout");
			
		return false;
	};

/**
 * @method private Tree.prototype.toggle
 * 	This method is used to open or close the node depending on if the node is open or close.
 *  To improve performance, if the node is never be opened before, the code will construct its
 *	children's nodes and saved to the big array. So later on, when user clicks this node again,
 *  the children's node can be got directly from that big array.
 * @param int itemID
 *	The position of the node in the whole big array which includes all the rendered
 *	nodes.
**/
	Tree.prototype.toggle = function(itemID, nativeEvt)
	{

		if(itemID == 0)	return;


			//Call the onNodeExpand and onExpand event no matter if it has child or not. It is developer's responsibility to decide to call
			//server side data fetch or not.

		var itemObj = this.a_index[itemID];

		if(!itemObj.isOpened){
			//alert("we're opening in Toggle!");
			//expand event
			if(!itemObj.activeNodeEvent("onExpand",null,nativeEvt))
				return false;
			if(!itemObj.activeTreeEvent("onNodeExpand",null,nativeEvt))
				return false;


		}else{
			//collapse event
			if(!itemObj.activeNodeEvent("onCollapse",null,nativeEvt))
				return false;
			if(!itemObj.activeTreeEvent("onNodeCollapse",null,nativeEvt))
				return false;



		}

		this.getChildrenItems(itemObj);
		return false;

	};
/**
 * @method private Tree.prototype.contextmenu
 * 	This method is a call back method which right now is not used
 * @param int itemID
 *	The position of the node in the whole big array which includes all the rendered
 *	nodes.
**/
	Tree.prototype.contextmenu = function(itemID)
	{
		var itemObj = this.a_index[itemID];
		if (this.oncontextmenu != null)
			this.oncontextmenu(itemObj);
	};
/**
 * @method private Tree.prototype.expand
 * 	This method is used to open the specified node
 * @param int itemID
 *	The position of the node in the whole big array which includes all the rendered
 *	nodes.
**/
	Tree.prototype.expand = function(itemID)
	{


		var itemObj = this.a_index[itemID];
		if(!itemObj.isOpened)
			itemObj.open(itemObj.isOpened);

	};

/**
 * @method private Tree.prototype.getChildrenItems
 * 	This method is used to get the children nodes of the specified node
 * @param obj itemObj
 *	The parent object
 * @param boolean expandFlag
 *	indicate this node is expand or collapse
**/

	Tree.prototype.getChildrenItems = function(itemObj,expandFlag)
	{
		if(itemObj.childItems!=null&&itemObj.childItems.length>0&&itemObj.childItems[itemObj.childItems.length-1]!=null){
			//this node is already rendered.

			if(expandFlag=="expand"&&itemObj.isOpened)
			{
				return;
			}
			else if(expandFlag=="collapse"&&!itemObj.isOpened)
			{
				return;
			}
			else
			{
				if(this.crumtree)
				{
					//close other open node
					for(var i=1; i<this.a_index.length; i++)
					{
						var tempObj = this.a_index[i];

						 if(tempObj.isOpened)
						 {
							 tempObj.open(true);
							 break;

						 }

					}
				}
				
				itemObj.open(itemObj.isOpened);
			}
		}
		else
		{
			var nIndex =0;
			var xmlNode = itemObj.xmlNode;
			var eobj = itemObj.eobj;
			var elementWrap = new ElementWrapper(xmlNode, true);
			this.Adapter.GetChildContent(elementWrap, eobj, 1);

			var preNodelen = this.alength;
			for (var i = 0; i < xmlNode.childNodes.length; i++)
			{
				var node = xmlNode.childNodes[i];
				if (node.nodeType != 1)
					continue;
				new TreeItem(itemObj, node, nIndex);
				nIndex++;
			}
			if(this.crumtree)
			{
					for(var i=1; i<this.a_index.length; i++)
					{
						var tempObj = this.a_index[i];
						if(tempObj.isOpened)
						{
							tempObj.open(true);
							break;
						}
					}
			}

			itemObj.open(itemObj.isOpened);
			this.Adapter.SetPropertyBinders(preNodelen);

			if(this.enableSelect)
			{
				if(this.checkedItemArray[itemObj.eobj.ID]==null)
				{
					itemObj.check('ONUNSELECT',true)
				}
				else
				{
					itemObj.check('ONSELECT',true);
				}
			}
		}
	}

/**
 * @method public Tree.prototype.expandAll
 * 	This method is used to open all the children nodes of the specified node
 * @param Object eobject
 *	The expanded node's eobject.  If omitted, the root (and all children) are expanded.
**/
	Tree.prototype.expandAll = function (eobj)
	{
		if (this.crumtree) return;
		var i,itemObj,eobject = (eobj)?eobj:this.a_index[0].eobj;
		
		for(i=0; i<this.a_index.length; i++)
		{
			itemObj = this.a_index[i];
			if(eobject.ID==itemObj.eobj.ID){
				itemObj = this.a_index[i];
				//expand event
				if(!itemObj.activeNodeEvent("onExpand","-1"))
					return false;
				if(!itemObj.activeTreeEvent("onNodeExpand","-1"))
					return false;

				this.expandAllInternal(i);
				break;
			}
		}
	}
/**
 * @method private Tree.prototype.expandAllInternal
 * 	This method is used to open all the children nodes of the specified node
 * @param int itemID
 *	The position of the node in the whole big array which includes all the rendered
 *	nodes.
**/
	Tree.prototype.expandAllInternal = function (itemID)
	{
		var j, itemObj = this.a_index[itemID];

		this.getChildrenItems(itemObj,"expand");
		for (j = 0; j < itemObj.childItems.length; j++)
		{
			if (itemObj.childItems && itemObj.childItems[j])
				this.expandAllInternal(itemObj.childItems[j].itemID);
		}
	}





/**
 * @method public Tree.prototype.collapseAll
 * 	This method is used to collapse all the children nodes of the
 *	specified node
 * @param Object eobject
 *	The expanded node's eobject.  If null, first level (not root) of tree is collapsed.
    To collapse all (including the root) pass the root's eobject as the parameter
**/
	Tree.prototype.collapseAll = function (eobject)
	{
		if(this.crumtree) return;
		var i,itemObj;
		// If we have an eobj, then use collapse it
		for(i=0; i<this.a_index.length; i++)
		{
			itemObj = this.a_index[i];
			if((eobject==null && i!=0) || (eobject!=null && eobject.ID==itemObj.eobj.ID)){
				//collapse event
				if(!itemObj.activeNodeEvent("onCollapse", "-1"))
					return false;
				if(!itemObj.activeTreeEvent("onNodeCollapse", "-1"))
					return false;

				this.collapseAllInternal(i);
				if (eobject!=null) break;
			}
		}
	}
/**
 * @method private Tree.prototype.collapseAllInternal
 * 	This method is used to open all the children nodes of the specified node
 * @param int itemID
 *	The position of the node in the whole big array which includes all the rendered
 *	nodes.
**/
	Tree.prototype.collapseAllInternal = function (itemID)
	{
		var j, itemObj = this.a_index[itemID];

		this.getChildrenItems(itemObj,"collapse");

		for (j = 0; j < itemObj.childItems.length; j++)
		{
			if (itemObj.childItems && itemObj.childItems[j])
				this.collapseAllInternal(itemObj.childItems[j].itemID);
		}
	}




/**
 * @method private Tree.prototype.mout
 * 	This method is used to update the status in the browser when user's mouse out of that node
 * @param int itemID
 *	The position of the node in the whole big array which includes all the rendered
 *	nodes.
**/
	Tree.prototype.mout  = function (itemID)
	{
		if(this.a_index[itemID])
			this.a_index[itemID].upstatus(true,"mouseout")
	};

/**
 * @method private Tree.prototype.mover
 * 	This method is used to update the status in the browser when user's mouse over that node
 * @param int itemID
 *	The position of the node in the whole big array which includes all the rendered nodes
 *	nodes.
**/
	Tree.prototype.mover = function (itemID)
	{
		if(this.a_index[itemID])
			this.a_index[itemID].upstatus(false,"mouseover")
	};

/**
 * @method private Tree.prototype.dover
 * 	This method is used to update the status in the browser when user's mouse drags something over a node
 * @param int itemID
 *	The position of the node in the whole big array which includes all the rendered nodes
 * @param event e
 *	If mozilla browser this contains the event object. For IE this is null
 * 
**/
	Tree.prototype.dover = function (itemID, e)
	{
		//If mozilla set myEvent to e otherwise set it to window.event
		var nativeEvent=e?e:window.event; 
		var itemObj = this.a_index[itemID];
		var r = false;

		if (itemObj == null)
			return;

	  	// Default value of controlHighlight flag is false if this is the case the highlighting is carried out here
		if(this.userHighlight==false)
		{
			Log.debug("Tree.DOVER()","tree control is controling the highlighting");		
			var treeResult=itemObj.activeTreeEvent("onNodeDragEnterOver", nativeEvent);
			var nodeResult=itemObj.activeNodeEvent("onDragEnterOver", nativeEvent);
		
			//if event is dragenter and return value is true highlighting should occur
			if(treeResult==true || nodeResult==true)
			{
				//set style for the event
				if(this.a_index[itemID])
					this.a_index[itemID].upstatus(false,"dragover");
			}
		}
		//Otherwise user controls highlight in external methods
		else
		{
			Log.debug("Tree.DOVER()","we're controling the highlighting this.rootItem.controlHighlight is "+this.rootItem.controlHighlight);	
			if (itemObj.activeTreeEvent("onNodeDragEnterOver", nativeEvent)) {
				if(itemObj.activeNodeEvent("onDragEnterOver", nativeEvent))
					r = true;
			}
		}
	// If this is IE we must return false AND we need to cancel the event
	if (isIE() && !r) {
		this.CancelEvent(nativeEvent);
		return false;
	}
	return;
	};

/**
 * @method private Tree.prototype.CancelEvent
 * 	This method is used to call cancel event
 * @param int itemID
 *	The position of the node in the whole big array which includes all the rendered
 *	nodes.
**/
Tree.prototype.CancelEvent = function(e)
	{
//		Log.debug("TreeControl.j cancelEvent()"," we're in CancelEvent");
		if (e==null) {
			if (isIE()) window.event.returnValue = false;
		} else {
			if (e.preventDefault) 
				e.preventDefault();
			else
				e.returnValue = false;
		}
	}

/**
 * @method private Tree.prototype.dout
 * 	This method is used to update the status in the browser when a mouse containing drag info leaves a node
 * @param int itemID
 *	The position of the node in the whole big array which includes all the rendered
 *	nodes.
**/
	Tree.prototype.dout = function (itemID)
	{
		if(this.a_index[itemID])
			this.a_index[itemID].upstatus(true,"dragout");
	};
	
/**
 * @method public Tree.prototype.restoreUIState
 * 	This function is used to restore the UI state of the tree based
 *	on a state string.
 * @param String stateString
 *  A comma delimited signature string to represent which nodes are
 *	open, such as 1,5,8,9:10:1,2,3,4, indicating that nodes with object
 *	signatures: 1, 5, 8, 9 are expaned. 10 is highlighted.1,2,3,4 are selected.
 * @param String highlightName 
 * 	The itemName attribute of an object of type treeItem. If calling from an event 
  * handler the parameter would be thisObj.itemName 
**/
    Tree.prototype.restoreUIState = function(stateString, highlightName)
	{

	  	if(stateString == null || stateString == "" ||stateString == "null;null;null")
	  	{
	     	return;
	  	}
 		//Timer.add('enter restoreUIState');
	  
		var stateArray=null;
		var highlightNode = null; 
		var selectArray = null;
      
	    //If there is a highlight node included on the statestring
	    if(stateString.indexOf(";")!=-1)
	    {
			//Create an array split by the statestrings ';' delimiter.
			//There are three possible element types here.
			var openhighlightArray = stateString.split(";");
			
			//1. The first element contains a sting with a list of expanded nodes
			//Split the string at the ',' to create a state array
			stateArray = openhighlightArray[0].split(",");
			
			//2. This element contains a single node that is highlighted.
			//This represents the treeItem using treeItem.itemName attribute
			highlightNode = openhighlightArray[1];
			
			//3.The third is the info for selected arrays (selected check boxes)
			// If it exists split the string at the ',' to create a selected items array
			if(openhighlightArray[2]!=null&&openhighlightArray[2]!="")
					selectArray = openhighlightArray[2].split(",");
		}
		//If no highlight/select info its just a case of making an array from the statestring 
		//using its comma delimiters
		else
		{
			stateArray = stateString.split(",");
		}
			
		//openLen is the stateArray length - 1 
 		var openLen = stateArray.length-1;
 		
 		//For each element on the stateArray (nodes that should be expanded)
 		for (var j = 0; j <=openLen; j++)
 		{
 			//Get the signature of the node
 		   	var signature = stateArray[j];
 		   	
 		   	//For each node on the expandedNode array
			for(var i = 0; i< this.Adapter.expandedNode.length; i++)
			{
				//Calculate  signature only once.
				//We are not supposed to update key fields.
				//Otherwise we could get an out-of-date signature.
				
				//Get the signature of the node on the expanded array
				var treeNode = this.Adapter.expandedNode[i];
				var tempSignature = treeNode.getSignature();

				//If the signature of the node on the statestring is also 
				//on the expanded array
				if(tempSignature != null && tempSignature == signature)
				{						
					//Expand node
					var XMIId = this.Adapter.expandedNode[i].ID;
					this.expandNodeWithID(XMIId,j,openLen);
	
					//exit the inner loop
					break;
				}//End if tempSignature = signature
			}//End for each node on the expandedNode array
		}//End for each element on the stateArray 
		
		
		for(var i=0; i<this.a_index.length; i++)
	  	{
			var treeNode = this.a_index[i].eobj;
			var tempSignature = treeNode.getSignature();
			//if a node was specified to be highlighted by the methods second parameter 
			if(highlightName!=null&&highlightName!='undefined')
			{
				//Get display name for node
				var displayName = this.Adapter.applyStyle(treeNode.EClass.Name , treeNode);
				//If the display name equals the name of the node to be highlighted
				if(displayName.name == highlightName)
				{
					//The highlight node is now the node specified by the second parameter
					highlightNode = tempSignature;
					//These too line were put here with the comment "so skip the rest of open node;"
					//I don't think a node being highlighted should be reason for the rest of state 
					//string to be ignored
					//j=stateArray.length; 
					//break;
				}
			}//End if highlightName exists
				
			if(highlightNode!=null)
  			{
				if(tempSignature != null && tempSignature == highlightNode)
				{
					if(highlightName!=null&&highlightName!='undefined')
					{
						var displayName = this.Adapter.applyStyle(treeNode.EClass.Name , treeNode);

						if(displayName.name == highlightName)
						{
							this.a_index[i].select(false, true);
							break;
						}
						else
						{
							var xmiId = treeNode.ID;
							this.expandNodeWithID(xmiId);
							var parentNode = this.a_index[i];
							for(var j=0; j<parentNode.childItems.length; j++)
							{
								var highlightNode = parentNode.childItems[j];
								displayName = this.Adapter.applyStyle(highlightNode.eobj.EClass.Name , highlightNode.eobj);
								if(displayName.name == highlightName)
								{
									//in restoreUIState, don't call highlight event. Only highlight the node.
									highlightNode.select(false,true);
									break;
								}
							}//end for
						}//end else
					}//end if highlightName exists
					else
					{
						this.a_index[i].select(false,true);
						break;
					}
				}//end if tempsignature = highlightnode
			}//end if(highlightNode!=null)
		}//for each node on the tree (in a_index)
		
		if(highlightNode!=null)
		{
			//Set current highlighted (named selected) item for the tree to be the node 
			//specified as the node to be highlighted in the parameters passed to this method
			if(!this.selectedItem)
				this.selectedItem = highlightNode;
	  	}//if(highlightNode!=null)


		//If there is a list of selected nodes
		if(selectArray!=null)
		{
			//For each item in the selected array
			for (var j = 0; j < selectArray.length; j++)
		  	{
				/*
			 	var signature = selectArray[j];
			 	if(signature!=null&&signature!="null"&&Trim(signature)!="")
					this.a_index[selectArray[j]].check("ONSELECT",true);
				*/
				var signature = selectArray[j];
				//Set the node to be selected
				for(var k=0; k<this.a_index.length;k++)
				{
					var tempSignature = this.a_index[k].eobj.getSignature();
					if(tempSignature != null && tempSignature == signature)
					{
						this.a_index[k].check("ONSELECT",true);
						break;
					}
				}//End for
			}//End for
		}//End if selectArray
	}


/**
* @method private Tree.prototype.getStateObj
*   This function is used to get the state object of the tree
**/

	Tree.prototype.getStateObj = function()
	{
		var stateObj = new Object();

		var openString= selectString = "";
		
		//Fix provided by Chuck Dumount
		//Was previously for/in stmt
		for(var key=0; key<this.a_index.length; key++)
		{
			var itemObj = this.a_index[key];

			 if(itemObj.isOpened == true)
			 {
				 openString = constructStateString(itemObj.eobj, openString);
				//alert("inside loop stateString:" + openString);
			 }
			 if(this.rootItem.checkedItemArray[itemObj.eobj.ID]!=null)
			 {

			 	selectString = constructStateString(itemObj.eobj,selectString);
		 	 }

		}
		stateObj.openStr = openString;
		stateObj.selectStr = ";"+selectString;

		return stateObj;

	}
/**
* @method private Tree.prototype.getSelStateStr
*   This function is used to get the selection state string of the tree
**/

	Tree.prototype.getSelStateStr = function()
	{


      if(this.selectedItem){

		  var signature = (typeof(this.selectedItem)=="string")?this.selectedItem:this.selectedItem.eobj.getSignature();
		  if(signature!=null)
		  {
			return ";"+ signature;
		  }


	  }else{

		  return ";null";
	  }

	}

/**
* @method public Tree.prototype.generateStateString
* This function is used to generate a state string based on the
*	signature of the eobject on  each node. A comma delimited signature
*	string to represent which nodes are open, such as 1,5,8,9:10:1,2,3,4,
*	indicating that nodes with object signatures: 1, 5, 8, 9 are expaned.
*	10 is highlighted.1,2,3,4 are selected.
*
**/

   Tree.prototype.generateUIStateString = function()
	{
      var stateString="";

      var stateObj = this.getStateObj();

       //add highlight information to state string, required by biz porlet team Mike and Cesar.
      //use ":" to delimit open node state information and highlight node information.
		stateString += stateObj.openStr;

		stateString += this.getSelStateStr();

		stateString += stateObj.selectStr;

      //alert("inside generateTreeStateString stateString:" + stateString);

      return stateString;
	}
/**
* @method private generateStateString
* this method is used to construct open or selected nodes states string
* @param Eobject eobj
* 	The eobject of the node
* @param String stateString
*	The state string of the node
**/
	function constructStateString(eobj, stateString)
	{

				var signature = null;
				if(eobj) signature= eobj.getSignature();

				if(signature!=null)
				{
					if(stateString=="")
					{
						stateString= signature;
					}
					else
					{
						stateString+=","+signature;
					}
				}
				return stateString;


	}

/**
* @method private expandNodeWithID
* This function is used to restore the UI state of the tree based on a
*	state string.
* @param String xmiID
*    The xmiID of the EObject which need to be opened.
* @param Integer index
*    The position of the expanded node in the stateString
* @param Integer len
*	The length of the expanded node
**/

	Tree.prototype.expandNodeWithID = function(xmiID, index, len)
	{


		for(var i=0; i<this.a_index.length;i++){

			if(this.a_index[i].eobj.ID == xmiID){

				var expandNode = this.a_index[i];

				var xmlNode = expandNode.xmlNode;
				var nIndex= 0;



				var eobj = expandNode.eobj;


				var elementWrap = new ElementWrapper(xmlNode, true);

				if(xmlNode.getAttribute("childrenLen")>xmlNode.childNodes.length){
					//need to construct the next level of xml data


					this.Adapter.GetChildContent(elementWrap, eobj, 0);


				}
			//	alert(elementWrap.element.getAttribute("childrenLen"));

				//alert(GetXMLString(10,xmlNode));

				//new TreeItem(itemObj, xmlNode, 0, this.IconLib);

				if(elementWrap.element.getAttribute("childrenLen")>0){
					//has children nodes
					for (var j= 0; j < xmlNode.childNodes.length; j++)
					{
						var node = xmlNode.childNodes[j];

						if (node.nodeType != 1)
							continue;


						new TreeItem(expandNode, node, nIndex);

						nIndex++;

					}


					this.expand(i);

               //alert("after expand");
				/*	//do onExpand event handler for the last expanded node.
					if(index==len){


						//if(!expandNode.isOpened){
							//expand event
							if(!expandNode.activeNodeEvent("onExpand"))
								return false;
							if(!expandNode.activeTreeEvent("onNodeExpand"))
								return false;


					}*/



				}else{

					//change the + to leaf
					expandNode.childItems.length = 0;
					expandNode.changeNode2Leaf();
					expandNode.isOpened = true;


				}
				break;
			}//if
			//break;

		}


	}
	/**
	* @method public Tree.prototype.constructStateStrFromObj
    * 	This function is used to generate the state string from an array of
    *	eobjects, Tracy requested.
    * @param Array inputArr
    *   The array of eobjects starting from root to the to-be-highlighted
    *	nodes. for example,
    *	&nbsp;&nbsp;root
    *	&nbsp;&nbsp;&nbsp;|___dir1
    *	&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;|__dir12
    *	&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;|___file1
    *	&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;|___file2
    *   &nbsp;&nbsp;&nbsp;|____dir2
    *	If you want file1 to be highlighted at the first render time, so the
    *	eobjects array will look like this, [root.eobject, dir1.eobject,
    *	dir12.eobject, file1.eobject]
    * @param String stateStr
    * State string
    * @return String stateStr
    *	The state string which can be used to restore the status.
    **/

	Tree.prototype.constructStateStrFromObj = function(inputArr, stateStr)
	{

		if(inputArr==null||inputArr=='undefined')
			return null;

		var stateString = openString = "";
		var len = inputArr.length-1;

		for(var i=0; i<len; i++)
		{

			openString = constructStateString(inputArr[i], openString);

		}

		stateString += openString;

		if(inputArr[len]!=null){

			var signature = inputArr[len].getSignature();
			if(signature!=null)
			{
				stateString += ";"+ signature+";";
			}


		}else{

		    stateString += ";null;";
		}



		return stateString

	}



	/**
 	* @method public Tree.prototype.constructStateStrFromAttr
    * 	This function is used to generate the state string from attribute
    * 	name and an array of attribute value, Darryl requested.
    * @param String attrName
    *  The name of the attribute of eobject.like "UID".
    * @param Array valueArr
    *    The array of value of attribute of eobjects starting from root
    * 	 to the to-be-highlighted nodes. for example,
    *	&nbsp;&nbsp;root
    *	&nbsp;&nbsp;&nbsp;|___dir1
    *	&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;|__dir12
    *	&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;|___file1
    *	&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;|___file2
    *   &nbsp;&nbsp;&nbsp;|____dir2
    *	If you want file1 to be highlighted at the first render time, so
    *	the array will look like this, ["root", "dir1", "dir12", "file1"]
    * @param String stateStr
    *  The state string
    * @return String stateStr
    *	The state string which can be used to restore the status.
    **/

	Tree.prototype.constructStateStrFromAttr = function(attrName, valueArr,stateStr)
	{
		var eobjArr= [];

		if(attrName&&valueArr)
		{
			for(var i=0; i<valueArr.length; i++)
			{
				eobjArr[eobjArr.length]= findEObjectByAtrName(this.Adapter.eobjRoot, attrName, valueArr[i]);
			}
		}

		if(eobjArr.length>0)
			return this.constructStateStrFromObj(eobjArr, stateStr);
		else
			return null;

	}


/**
* @method public Tree.prototype.getSelectedItems
*   This function is returning an array of selected item's eobjets. If none
*	of items is selected, then return empty array
* @return Array
*	An array of selected item's eobjects
**/
	Tree.prototype.getSelectedItems = function()
	{

		return this.compactCheckedItemsArray();
	}


/**
* @method public Tree.prototype.getHighlightedItem
*   This function is returning the highlighted item's eobject. If none of items
*	is highlighted, then return null.
* @return eObject
*	the eObject of highlighted item.
**/
	Tree.prototype.getHighlightedItem = function()
	{
		return 	this.selectedItem.eobj;

	}



/**
* @method public Tree.prototype.getExpandededItems
*   This function is returning the expanded item's eobject. If none of items
*	is expanded, then return null.
* @return Array
*	An array of expanded item's eobjects
**/
	Tree.prototype.getExpandededItems = function()
	{
		return 	this.compactOpenItemsArray();

	}


/**
 * @method private Tree.prototype.compactCheckedItemsArray
 * 	This method is used to clean up the checked items array so that the event object will only carry the checked node's eobjects.
**/
	Tree.prototype.compactCheckedItemsArray = function()
	{
		var checkedItems = this.checkedEObjectArray, rtArray = [], k;
		for(k in checkedItems)
		{
			if(checkedItems[k] != null)
			{
				rtArray[rtArray.length] = checkedItems[k];
			}

		}
		return rtArray;
	}


/**
 * @method private Tree.prototype.compactOpenItemsArray
 * 	This method is used to clean up the open items array so that the event object will only carry the checked node's eobjects.
**/
	Tree.prototype.compactOpenItemsArray = function()
	{
		var openItems = this.Adapter.expandedNode, rtArray = [], k;
		for(k in openItems)
		{
			if(openItems[k] != null)
			{
				rtArray[rtArray.length] = openItems[k];
			}

		}
		return rtArray;
	}
	
/**
* @method public TreeControl.prototype.addDOMElem Adds a newly created element to the DOM. 
* 	Currently used by TreeAdapter.addNode. This was moved to the tree control in order to 
* 	keep all creation of DOM elements in one place
* @param eobject parentIndex
*	Index to provide location of parent on a_index array	
* @param int domElem 
*	Newly created domElement
**/
Tree.prototype.addDOMElem = function (parentIndex,domElem)
{
	//Add newly created element to the DOM
	var htmlElement = document.getElementById('i_div'+this.a_index[parentIndex].rootItem.itemID+'_' + this.a_index[parentIndex].itemID);
	if(htmlElement)
	{
		if(domElem[0] != null)
			htmlElement.appendChild(domElem[0]);
		if(domElem[1] != null)
			htmlElement.appendChild(domElem[1]);
	}
	this.Adapter.SetPropertyBinders(0);
	Log.debug("TreeAdapter.addNode() ","step 5 of method");	
}

/**
* @method public TreeControl.prototype.removeChildrenItems Removes the child items of a node from the tree control.
* 	(Should be called after removing them from the model using the eRemove method)
* @param eobject parentObj
*	Parent of children that are to be removed
**/
Tree.prototype.removeChildrenItems = function(parentObj)
{	
	if(parentObj.childItems!=null&&parentObj.childItems.length>0&&parentObj.childItems[0]!="dummy")
	{
		this.recursiveRemove(parentObj);
		parentObj.childItems[0]="dummy";
		
		//clean up a_index....remove null values
		var tempArray=[], tempLen=0, i;
		for(i=0; i<this.a_index.length; i++)
		{
			if(this.a_index[i]!=null)
			{
				tempArray[tempLen]=this.a_index[i];
				this.a_index[i].itemID=tempLen;
				tempLen++;
			}
		}
		this.a_index=tempArray;
	}
}

/**
* @method private TreeControl.prototype.recursiveRemove recursively removes all descendants from the
* 	a_index array and clears each parentnode's childItems array
* @param eobject parentObj
*	Index to provide location of parent on a_index array	
*/
Tree.prototype.recursiveRemove = function(parentObj)
{
	//If the node has children. This condition stops the recursive call
	if(parentObj.childItems!=null&&parentObj.childItems.length>0&&parentObj.childItems[0]!="dummy")
	{
		//For each child of the object
		for(var k=0; k< parentObj.childItems.length; k++)
		{			
			//If the child appears on the a_index array remove it. 
			for(var j=0;j<this.a_index.length;j++)
			{
				if(this.a_index[j]!=null)
				{
					if(parentObj.childItems[k].itemID==this.a_index[j].itemID)
						this.a_index[j]=null;
				}
			}
			//Remove all children for that child
			for(var i=0; i< parentObj.childItems.length; i++)
			{
				this.recursiveRemove(parentObj.childItems[i]);
			}
		
		}
		
		//Remove reference to children from parent object 
		parentObj.childItems[0]=null;
		parentObj.childItems.length=0;
	}
}

/**
 * @class public findItemByAttrName
 * Method to find a certain treeItem by finding its corresponding eobject. 
 * The eobject is gotten using attribute information
 * @param attrName
 *	 Name of the attribute belonging to the treeItems eObject
 * @param attrValue
 *	 Value of that attribute.
 **/
Tree.prototype.findItemByAtrName = function(attrName, attrValue)
{
	var root = this.Adapter.eobjRoot;
	var eobject = findEObjectByAtrName(this.Adapter.eobjRoot, attrName, attrValue);
	var item = null;
	if (eobject) {
		for(var j=0;j<this.a_index.length;j++)
		{	
			//find the treeItem corresponding to the eobject
			if(this.a_index[j].eobj.ID==eobject.ID)
			{
				item = this.a_index[j];
				break;
			}
		}
	}
	return item;
}

/**
 * @class private TreeItem
 * This class is used to construct every sigle nodes and put them into a big array based on the xml
 * which is getting from the tree adapter.
 * @constructor private TreeItem
 * @param Object parentItem
 *	The object which is the parent of the to be constructed tree node
 * @param String xmlNode
 *	The paretnItem's xml string which has all the information about children nodes
 * @param int itemIndex
 *	This number is used to indicate how many childrens node of the parent node
 * @param String AnIconLib
 *	The icons' relative path which is the same as urlprefix in the tree constructor
 * @param int isLeaf
 *	Flag used to allow user to explicitly specify if a node is a leaf prior to 
 *  knowing its children info. Default is 
 *  2 = default(not explicitly specified)
 *  1 = true(explicitly identified as leaf)
 *  0 = false (explicitly identified as NOT a leaf)
 *
**/
function TreeItem (parentItem, xmlNode, itemIndex, isLeaf)
{
	//If isLeaf is undefined set it to default value
	if(isLeaf=="undefined"||isLeaf==null)
		this.isLeaf=2;
	else
		this.isLeaf=isLeaf;

	this.currentRowNumber;
	this.currentCellNumber;

	// TreeItem properties
	this.depth  = parentItem.depth + 1;
	this.xmlNode = xmlNode;
	//this.itemName = this.xmlNode.nodeName;
	this.itemName = this.xmlNode.displayName;
	this.rootItem = parentItem.rootItem;
	this.parentItem = parentItem;
	this.itemIndex = itemIndex;
	//this.isOpened = !this.depth;
	this.isOpened = false; // initially, the item will always be closed, starting from the top.
	this.rootItem.alength++;

	this.itemID = this.rootItem.a_index.length;
	this.rootItem.a_index[this.itemID] = this;

	parentItem.childItems[itemIndex] = this;

	this.customItemOpenedImage = xmlNode.getAttribute("treeopenimgfile");
	this.customItemClosedImage = xmlNode.getAttribute("treecloseimgfile");
	this.showSystemIconFlag = xmlNode.getAttribute("showsystemiconflag");
	this.customOpenedImagealt = xmlNode.getAttribute("treeopenimgalt");
	this.customClosedImagealt =  xmlNode.getAttribute("treecloseimgalt");
	//this.EObjectID = xmlNode.getAttribute("EObjectID");

	this.eobj = xmlNode.eobject;
	//this.itemID = this.eobj.getSignature();
	//this.rootItem.a_index[this.itemID] = this;
	

	this.propertyName = xmlNode.propertyName;

	//EventHandler should be filled with either events or behaviors.
	this.eventHandler = ""; //node's event handler
	if(xmlNode.eventHandler!=""&&xmlNode.eventHandler!=null)
		this.eventHandler=xmlNode.eventHandler;
	else if(xmlNode.behaviorHandler!=""&&xmlNode.behaviorHandler!=null)
		this.eventHandler=xmlNode.behaviorHandler;

	//Setting up child information from limited information available
	//First two options are used in the dynamic building of a tree when the
	//child information for the node is not available at node creation time. 
	//In this situation the developer must specify explicitly whether a node
	//should be a leaf or not. This flag is sent in via treeAdapter.addNode method 
	//in the last parameter
	
	//Variable containing info regarding children
	var numChildren = xmlNode.getAttribute("childrenLen");
	
	//If explicitly specified as leaf at creation time
	if(this.isLeaf==1)
		this.childItems = [];
	//If explicitly specified as NOT a leaf and if xmlNode.getAttribute("childrenLen") returns a 
	//number greater than 0
	else if(this.isLeaf==0)
	{	
		if(numChildren!=null&&numChildren>=0)
		{
			var dummyObject="dummy";
			this.childItems = new Array(1); 
			//need to do more than give it a length because if there is no value for childItems[0] 
			//it will be shown as a leaf 
			this.childItems[0]=dummyObject;
		}
	}	
	//When NOT creating dynamically it is determined whether it has children or not is based on the value provided by 
	//the variable numChildren which contains the value returned by xmlNode.getAttribute("childrenLen")
	else if( ( numChildren!=null && numChildren>0 )
			|| this.itemID==0 )
		this.childItems = new Array(1); 
	//If no valid value is returned by xmlNode.getAttribute("childrenLen") then it is considered a leaf.
	else
	{
		xmlNode.setAttribute("hasNoChildrenValues",true);
		this.childItems = [];
	}

	var node, i, nIndex = 0;
	for (i = 0; i < xmlNode.childNodes.length; i++)
	{
		node = xmlNode.childNodes[i];

		if (node.nodeType != 1)
			continue;
		
		new TreeItem(this, node, nIndex);

		nIndex++;
	}

	//Flag to indicate if the node has any related expand or collapse events
	var expandCheck=this.checkForEvent("onnodeexpand","onexpand");
	var collapseCheck=this.checkForEvent("onnodecollapse","oncollapse");
	if(expandCheck||collapseCheck)
		this.expandCollapseCheck=true;
	else
		this.expandCollapseCheck=false;
		
	//Flag to indicate whether a nodes parent is the rootnode or not
	if(this.parentItem.itemID==0)
		this.isRootChild="true";
	else
		this.isRootChild="false";
		
}

/**
 * @method private get_E
 * 	This is an utility function which is is used to get the html object based on the tag id
 * @param String s_id
 *	html tag id
**/
	TreeItem.prototype.getE = (document.all) ?	
		function (s_id) { return document.all[s_id] } :
		function (s_id) { return document.getElementById(s_id) };

/**
 * @method private TreeItem.prototype.checkForEvent
 * 	Checks for existance of event on a tree node
 * @parameters 
 *  treeEvent: the tree event you are looking to verify e.g."onnodehighlight"
 *  nodeEvent: the node event you are looking to verify e.g. "onhighlight"
 * @return boolean
 *	true indicates its presence.
 *
**/
TreeItem.prototype.checkForEvent = function (treeEvent,nodeEvent)
{
	var treeEvents=null, nodeEvents=null;
		
	//Check if node has an onnodeexpand or onnodecollapse event related to it at tree level
	if(treeEvent!=null&&treeEvent!="undefined"&&this.rootItem.eventArray.length>0)
		treeEvents=this.checkTreeEvent(treeEvent);

	//Check if node has an onexpand or oncollapse event related to it at a node level
	if(nodeEvent!=null&&nodeEvent!="undefined"&&this.eventHandler!=null&&this.eventHandler!='undefined')
		nodeEvents=this.checkNodeEvent(nodeEvent);

	//If any of these exist for a node set flag to true
	if(treeEvents!=null||nodeEvents!=null)
		return true;
	
	return false;
}

/**
 * @method private TreeItem.prototype.getJunctionImage
 * 	This method is used to get the images which connect between the nodes.  These images are sysstem icons.
 * @return String srcImage
 *	the image's source path
 *
**/
	TreeItem.prototype.getJunctionImage = function getJunctionImage()
	{
		var srcImage = '';
		
		//If there are no instantiated children, then a folder which has 
		//childItems.length > 0 should not have +/- 
		var childless = !this.hasChildren();
		
		//If its not a workplace tree +,- and joins shoul be shown for nodes at every level of the tree and for
		//every type of relationship
		if(this.rootItem.workPlaceFlag!=true)	
		{
			if (this.childItems.length && this.isOpened && !childless )
					srcImage = (!this.rootItem.rootVisibleFlag&&this.itemID==1)?this.rootItem.treeIconObjs["minustop"]:(this.isLast() ? this.rootItem.treeIconObjs["minus"] : this.rootItem.treeIconObjs["minusbottom"]);// junction for open node
				else if (this.childItems.length && !childless )
					srcImage = (!this.rootItem.rootVisibleFlag&&this.itemID==1)?this.rootItem.treeIconObjs["plustop"]:(this.isLast() ? this.rootItem.treeIconObjs["plus"] : this.rootItem.treeIconObjs["plusbottom"]);// junction for closed node
				else
					srcImage = (!this.rootItem.rootVisibleFlag&&this.itemID==1)?this.rootItem.treeIconObjs["jointop"]:(this.isLast() ? this.rootItem.treeIconObjs["joint"] : this.rootItem.treeIconObjs["jointbottom"]);// junction for leaf
			
		}
		//If it is a workplace tree the only parent/child relationship should be shown and not that of the siblings for the first level of nodes.
		else
		{
				//Sibling Relationships are maintained for any node if the root is visible and 
				//if the root is not wisible they are mantained for nodes that are not in the first level of children.
				if(this.isRootChild=="false"||this.rootItem.rootVisibleFlag)
				{
					if (this.childItems.length && this.isOpened && !childless )
						srcImage = (!this.rootItem.rootVisibleFlag&&this.itemID==1)?this.rootItem.treeIconObjs["minustop"]:(this.isLast() ? this.rootItem.treeIconObjs["minus"] : this.rootItem.treeIconObjs["minusbottom"]);// junction for open node
					else if (this.childItems.length && !childless )
						srcImage = (!this.rootItem.rootVisibleFlag&&this.itemID==1)?this.rootItem.treeIconObjs["plustop"]:(this.isLast() ? this.rootItem.treeIconObjs["plus"] : this.rootItem.treeIconObjs["plusbottom"]);// junction for closed node
					else
						srcImage = (!this.rootItem.rootVisibleFlag&&this.itemID==1)?this.rootItem.treeIconObjs["jointop"]:(this.isLast() ? this.rootItem.treeIconObjs["joint"] : this.rootItem.treeIconObjs["jointbottom"]);// junction for leaf
				}
				
				//lines are removed from between the siblings for nodes that are children of the root if the root is hidden
				else
				{
					if (this.childItems.length && this.isOpened && !childless )
						srcImage = this.rootItem.treeIconObjs["plainminus"];
					else if (this.childItems.length && !childless )
						srcImage = this.rootItem.treeIconObjs["plainplus"];// junction for closed node
					else
						srcImage = this.rootItem.treeIconObjs["empty"];// junction for node closed
				}
		}
		return srcImage.src;
	}
/**
 * @method private TreeItem.prototype.getBlankImage
 * 	Provide indenting for system icons (workplace request)
 * @return String srcImage
 *	the image's source path
 *
**/
TreeItem.prototype.getIndentImage = function getBlankImage()
{
	var srcImage = '';

	if(!this.rootItem.rootVisibleFlag)
		srcImage = this.rootItem.treeIconObjs["empty"];
	else
		srcImage = this.isLast() ? this.rootItem.treeIconObjs["joint"] : this.rootItem.treeIconObjs["jointbottom"];// junction for leaf
	
	return srcImage.src;
}	
	
/**
 * @method private TreeItem.prototype.getImage
 * 	This method is used to get node's images.
 * @param boolean isJunction
 *	this parameter is not needed right now.
 * @return String srcImage
 *	the image's source path
**/
	TreeItem.prototype.getImage = function getImage(isJunction)
	{
		var srcImage = '';

//code change for defect 267427(aix family)  so that root node icon will not be treated especially. 10/27
		if (this.depth >= 0)
		{
			var userDefinedIconMappingArray = null;
			//NS has to explicit to check if the image is not 'undefined'.
			if (this.customItemOpenedImage&& (this.customItemOpenedImage != 'undefined')&&(this.isOpened)) { //||(this.isSelected)) ) {
				// code to check the path?
				return this.customItemOpenedImage;


			} else if (this.customItemClosedImage&&(this.customItemOpenedImage != 'undefined')&& (!this.isOpened)) {
				// code to check the path?
				return this.customItemClosedImage;

			}

			//extra check added for root node introduced to resolve IIOSB defect 42767
			if (this.childItems.length && this.isOpened)
			{
				if(this.itemID ==0)
					srcImage = this.rootItem.treeIconObjs["baseopen"];
				else
				// folder icon opened
					srcImage = this.rootItem.treeIconObjs["nodeopen"];
			}
			
			else if (this.childItems.length)
			{
				if(this.itemID ==0)
					srcImage = this.rootItem.treeIconObjs["baseopen"];
				// folder icon normal
				else
					srcImage = this.rootItem.treeIconObjs["nodeclose"];
					
			}
			
			else 
			{
				if(this.itemID ==0)
					srcImage = this.rootItem.treeIconObjs["baseopen"];
				// leaf icon opened
				else
					srcImage = this.rootItem.treeIconObjs["leaf"];
			}
		}

		return srcImage.src;
	}

	TreeItem.prototype.refresh = function()
	{
		var wasOpened = this.isOpened;
		var htmlElement = this.getE('i_div' + this.rootItem.itemID + '_' + this.itemID);
		if (!htmlElement)
		{
			htmlElement = this.getE('i_txt' + this.rootItem.itemID + '_' + this.itemID);
			if (!htmlElement)
				return;
		}

		var htmlRootElement = this.getE('i_div' + this.rootItem.itemID + '_' + this.rootItem.itemID);
		if (!htmlRootElement)
		{
			htmlRootElement = this.getE('i_txt' + this.rootItem.itemID + '_' + this.rootItem.itemID);
			if (!htmlRootElement)
				return;
		}

		if (this.depth <= 1 && this.childItems.length > 0)
		{
		
			var tempElem = this.rootItem.childItems[0].init();			
			htmlRootElement.parentNode.appendChild(tempElem[0]);
			if(tempElem[1] != null)
				htmlRootElement.parentNode.appendChild(tempElem[1]);
			/*htmlRootElement.parentNode.innerHTML = this.rootItem.childItems[0].init();*/
			this.rootItem.childItems[0].open();
		}
		else
		{
			/*htmlElement.parentNode.innerHTML = this.init();*/
			var tempElem = this.init();			
			htmlElement.parentNode.appendChild(tempElem[0]);
			if(tempElem[1] != null)
				htmlElement.parentNode.appendChild(tempElem[1]);
		}

		this.open(!wasOpened);
	}



	/**
	 * @method private TreeItem.prototype.changeIcon
	 * 	This method is used to change the sytstem and folder icons
	**/
	TreeItem.prototype.changeIcon  = function item_open ()
	{
			
		/*var o_jicon = document.images['j_img' + this.rootItem.itemID + '_' + this.itemID],
			o_iicon = document.images['i_img' + this.rootItem.itemID + '_' + this.itemID];*/
		
		var o_jicon = document.getElementById("j_img" + this.rootItem.itemID + "_" + this.itemID);
		var	o_iicon = document.getElementById("i_img" + this.rootItem.itemID + "_" + this.itemID);


         if (o_jicon)
            {
               o_jicon.src = this.getJunctionImage();
               o_jicon.alt = this.getJunctionImageAlt();
               //Title attribute provides cross browser tooltips
               o_jicon.title= this.getJunctionImageAlt();
            }
         if (o_iicon)
            {
               o_iicon.src = this.getImage();
               o_iicon.alt = this.getImageAlt();
               //Title attribute provides cross browser tooltips
               o_iicon.title = this.getImageAlt();
            }

	}

	/**
	 * @method private TreeItem.prototype.changeNode2Leaf
	 * 	This method is used to change the node icons to leaf icon, also disable the onclick/ondblclick event.
	**/

	TreeItem.prototype.changeNode2Leaf  = function ()
	{
		this.changeIcon();

		var o_jalink = document.getElementById('j_a' + this.rootItem.itemID + '_' + this.itemID),
			o_ialink = document.getElementById('i_a' + this.rootItem.itemID + '_' + this.itemID);
		if(o_jalink){

			o_jalink.onclick = o_jalink.ondblclick = "javascript:return;";
		}
		if(o_ialink){

			o_ialink.ondblclick="javascript:return;";
		}
	}

/**
 * @method private TreeItem.prototype.open
 * 	This method is used to open or close the node depending on the node's statues (opened or closed)
 * @param boolean toClose
 *	If this node is open or closed
**/
	TreeItem.prototype.open = function item_open (toClose)
	{
		var i, len, divElem = this.getE('i_div' + this.rootItem.itemID + '_' + this.itemID);
		if (!divElem)
			return;
		if(this.childItems!=null&&this.childItems.length>0&&this.childItems[0]!="dummy")
		{
			if (!divElem.innerHTML)
			{
				var childItems = [], str;
				
			//	try{
					for (i = 0; i < this.childItems.length; i++)
					{
						if(this.childItems[i]!=null)
							childItems[i]= this.childItems[i].init();
					}
			//	}catch(e){
					//alert(odcTreeNoChildErr);
				//	this.childItems.length=0;
					//this.changeNode2Leaf();
	
			//		return;
	
			//	}
				/*divElem.innerHTML = childItems.join('');*/
				
				for(len = 0; len < childItems.length ;len++)
				{
					if(childItems[len]!=null)
					{
						divElem.appendChild(childItems[len][0]);
						if(childItems[len][1] != null)
							divElem.appendChild(childItems[len][1]);
					}		
				}
			}
		}//End if(this.childItems[0]!="dummy")
		
		divElem.style.display = (toClose ? 'none' : 'block');
		
		if(this.rootItem.dynamicFlag==true)
		{
			if(toClose == true) // remove this guys childs from the DOM
			{
				//alert("divElem.childNodes.length "+divElem.childNodes.length);
				for(i = (divElem.childNodes.length - 1) ; i >= 0; i--)
				{	// plus you have to remove in reverse order 
					if(divElem.childNodes[i] != null)
					{
						divElem.removeChild(divElem.childNodes[i]);
					}
				}
				
				//treeAdapter.expandedNode[treeAdapter.expandedNode.length]=root;
			}
		
		}
		
		this.isOpened = !toClose;
		this.changeIcon();

		this.upstatus(false, "open");

/*

		// Expand event firing
		if (!toClose && this.rootItem.onexpand != null)
			this.rootItem.onexpand(this);
		// Collapse event firing
		if (toClose && this.rootItem.oncollapse != null)
			this.rootItem.oncollapse(this);
*/

	}

/**
 * @method public TreeItem.prototype.isExpanded
 * 	This method is used to determine if the folder is opened or closed.
 * @return boolean
 *	true if the folder is opened
 *
**/
TreeItem.prototype.isExpanded=function()
{
	return this.isOpened;
}

/**
 * @method public TreeItem.prototype.hasChildren
 * 	This method is used to determine if the folder has any child nodes
 * @return boolean
 *	true if the folder has children
 *
**/
	TreeItem.prototype.hasChildren = function() 
	{
		var childless = this.xmlNode.getAttribute("hasNoChildrenValues");
		//need to make sure eval is same in firefox and ie -> convert string to boolean
		if( (childless=="true") || (childless==true) )
			childless = true;
		else
			childless = false;
		return !childless;
	}

/**
 * @method public TreeItem.prototype.toggle
 * 	This method is used to open or close the node depending on if the node is open or close.
 *
**/
	TreeItem.prototype.toggle = function(nativeEvt) 
	{
		this.rootItem.toggle(this.itemID,nativeEvt);
	}

/**
 * @method private TreeItem.prototype.activeTreeEvent
 * 	This method is used to active the corresponding event when user select or highlight a tree node.
 * @param String action
 *	the actions tree supports, like onHighlight, onSelect
 * @param String extrainfo
 *	variable created to allow different actions to carry in extra info specific to their needs
 *  used within their individual cases
**/
	TreeItem.prototype.activeTreeEvent = function(action,extrainfo,nativeEvt)
	{
		var uaction = action.toUpperCase();
			
		//Checks that handlers exist for this event. Returns array of event and handlers
		//Max handlers is 2. This is if a behavior has an onactionfunction and a behaviorAction attribute
		var events=this.checkTreeEvent(uaction);

		if(events!=null)
		{
			//Create event object
			var e = new ODCEvent(action);
			e.eobject=this.eobj;
		 	e.model = this.rootItem.Adapter.xmiModel;
	
			switch(uaction)
			{
				case "ONNODEHIGHLIGHT":
			 		e.propertyName = this.propertyName;
			 		break;
			
			 	case "ONNODEEXPAND": case "ONNODECOLLAPSE":
			 		//alert("we're in onexpand in activeTreeEvent");
			 		e.levels = (extrainfo)?extrainfo:"1";
					e.openedItemsArray = this.rootItem.compactOpenItemsArray();
			
			 		var str = this.rootItem.getStateObj();
			 		var openStr = str.openStr;
			 		var selStr = this.rootItem.getSelStateStr();
			 		var chkStr = str.selectStr;
			 		var lastExpandSig = this.eobj.getSignature();
			
			 		e.stateString = openStr+","+ lastExpandSig+selStr+chkStr;
			 		//alert(e.stateString);
			
			 		var expandArr = [];
			 		if(openStr.indexOf(",")!=-1)
			 			expandArr = openStr.split(",");
			 		else
			 			//in case only the root is expanded at this time.
			 			expandArr[expandArr.length] = openStr;
			
			 		expandArr[expandArr.length] = lastExpandSig;
			
			 		e.expandedNodeArray = expandArr;
			 		//alert("finishing in onexpand in activeTreeEvent");
			 		break;
			
			 	case "ONNODESELECT": case "ONNODEUNSELECT":
			 		e.checkedItemsArray = this.rootItem.compactCheckedItemsArray();
			 		break;
			
				case "ONNODEDROP": case "ONNODEDRAGENTEROVER":
					var nativeEvent=(extrainfo)?extrainfo:1;
			 		if(nativeEvent!=1) e.nativeEvent=extrainfo;
			 		e.propertyName = this.propertyName;
					break;
				
				default:
					break;
			}
			
			//Execute appropriate action
			for(var i=1;i<events.length;i++)
			{
				if(events[i]!=null && events[i]!="null" && events[i]!="undefined")
				{
					var handler = events[i];

					// If the handler is an object with a "behavior type", then this is a generic behavior
					if (typeof(handler) == "object" && handler.type!=undefined && handler.type=="generic")
					{
						// We must have a native event to do this
						if (hX_5 && nativeEvt!=null)
							hX_5.action.processAction(this.xmlNode, nativeEvt, null, handler.actions, true, handler.targets, null, null, null, null);
					}
					else
					{
						if (eval(handler+".call(this,this, e);")== false)
						{
							 return false;
						}
					}
					
					if(uaction=="ONNODEDRAGENTEROVER"&&!this.rootItem.userHighlight)
					{
						return "true";
		
		
					}
				}
			}	
		}
		return true;

	}
/**
 * @method private TreeItem.prototype.activeNodeEvent
 * 	This method is used to active the corresponding event when user select or highlight a tree node.
 * @param String action
 *	the actions tree node supports, like onHighlight, onSelect
 * @param String extrainfo
 *	variable created to allow different actions to carry in extra info specific to their needs
 *  used within their individual cases
**/
	TreeItem.prototype.activeNodeEvent = function(action,extrainfo,nativeEvt)
	{
		var uaction = action.toUpperCase();
		
		//Checks that handlers exist for this event. Returns array of events attached to a node
		var events=this.checkNodeEvent(uaction);

		if(events!=null)
		{	
			//Create event object
			var e = new ODCEvent(action);
			e.eobject=this.eobj;
			e.model = this.rootItem.Adapter.xmiModel;


			switch(uaction)
			{
				case "ONHIGHLIGHT":
			 		e.propertyName = this.propertyName;
					break;

			 	case "ONEXPAND": case "ONCOLLAPSE":
					e.openedItemsArray = this.rootItem.compactOpenItemsArray();
					e.levels = (extrainfo)?extrainfo:"1";
					break;

			 	case "ONSELECT": case "ONUNSELECT":
			 		e.checkedItemsArray = this.rootItem.compactCheckedItemsArray();
			 		break;

				case "ONDROP": case "ONDRAGENTEROVER":  
                    var nativeEvent=(extrainfo)?extrainfo:1;
                    if(nativeEvent!=1) e.nativeEvent=extrainfo;                
			 		e.propertyName = this.propertyName;
                    break;

				default:
					break;
			}
			
				
			//Execute appropriate action
			for(var i=1;i<events.length;i++)
			{	 		
				if(events[i]!="null"&&events[i]!="undefined")
				{
					var handler = events[i];
		 			
					// If the handler is an array, then this is a "generic behavior" of actions
					if (typeof(handler) == "object" && handler.length!=undefined && handler.push!=undefined)
					{
						// We must have a native event to do this
						if (hX_5 && nativeEvt!=null)
							hX_5.processBehavior(this.xmlNode, nativeEvt, handler[0], handler[1]);
					}
					else
					{
						if (eval(handler+".call(this,this, e);")== false)
						{
							 return false;
						}
					}
					
					if(uaction=="ONNODEDRAGENTEROVER"&&!this.rootItem.userHighlight)
					{
						return "true";
		
		
					}
				}//end if(events[i]!=null)
			}//End for each action triggered by event
			
		}//end if(events!=null)
	
		return true;

	}


	
/**
 * @method public TreeItem.prototype.checkTreeEvent
 * 	This method is used to check if a tree event exists as an eventHandler for that node. 
 	Returns relevent event array if it is a valid event with a corresponding handler
 * @param private String action
 *  The event type e.g onnodeexpand, onnodehighlight
 * @returns event array if appropriate otherwise returns null
**/
	TreeItem.prototype.checkTreeEvent = function (action)
	{
		var uaction = action.toUpperCase();
		if(this.rootItem.eventArray.length>0)
		{
			for(var i=0; i<this.rootItem.eventArray.length;i++)
			{
				if(this.rootItem.eventArray[i][0].toUpperCase() == uaction)
				{
					return this.rootItem.eventArray[i];
				}
        	}
        }
        return null;
        
     }       
     
/**
 * @method public TreeItem.prototype.checkNodeEvent
 * 	This method is used to check if a node event exists as an eventHandler for that node. 
 *	Returns relevent event array if it is a valid event with a corresponding handler
 * @param private String action
 *  The event type e.g ondrop, onclick
 * @ return event array if appropriate otherwise returns null
**/
	TreeItem.prototype.checkNodeEvent = function (action)
	{
		var uaction = action.toUpperCase();
		var i, eventArray = this.eventHandler.split("|"), returnArr=[];
		for(i=0; i<eventArray.length; i++) {
        	var tmpArr = eventArray[i].split(",");
        	//if the event is the one we are looking for get arr into correct state for activeNodeEvent
        	if(tmpArr[0].toUpperCase() == uaction) {
	        	var events=[];
	        	//Event type
	        	events[0]=tmpArr[0];
	        	//function if any
	        	events[1]=tmpArr[1].replace(/ /g, "");
	        	//There is a behaviorAction
				if(tmpArr.length==3)
					events[2]=[tmpArr[2],null];
	        	//There is a behaviorAction and a targetAction
	        	else if(tmpArr.length==4)
					events[2]=[tmpArr[2],tmpArr[3]];
	        	return events;
        	}
       	 }
	return null;
    }       
          
/**
 * @method public TreeItem.prototype.annotate
 * 	This method is used to append DOM elements to the text element within the table cell of a tree node label. 
 * @param String[] dataArray: Name/Value Pair array of the various attributes of the new element. Names are DOM 
 * 		attribute names except for two exceptions: 
 *				1. annotType: Must be used for the first entry in the dataArray. Specifies Type of new element
 *				2. nestedType: Used to identify nested elements, e.g text or image links where nesting occurs.
 * These names both must have one of the following values: text, link, image, input, span
 * @param private String nestedID
 *
**/
	TreeItem.prototype.annotate = function (dataArray,nestedID)
	{
			var divElem=null;
			if(!nestedID)
				divElem=this.getE('i_td' + this.rootItem.itemID + '_' + this.itemID);
			else
				divElem=this.getE(nestedID);

			//create new child element 
			var newElem;
			if(dataArray[1]=="text")
			{
				newElem=document.createTextNode(dataArray[3]);
			}
			else if(dataArray[1]=="link")
			{
				newElem = document.createElement("a");
				newElem.setAttribute("id","i_link" + this.rootItem.itemID + "_" + this.itemID);
			}
			else if(dataArray[1]=="image")
			{
				newElem = document.createElement("img");
				newElem.setAttribute("id","i_mg" + this.rootItem.itemID + "_" + this.itemID);
			}
			else if(dataArray[1]=="input")
			{
				newElem = document.createElement("INPUT");
				newElem.setAttribute("id","i_in" + this.rootItem.itemID + "_" + this.itemID);
			}
			else if(dataArray[1]=="span")
			{
				newElem = document.createElement("span");
				newElem.setAttribute("id","i_span" + this.rootItem.itemID + "_" + this.itemID);
			}
			else 
			{
				alert("Error: Unsupported annotation type");
				return;
			}
			
			//If not a nested type (link etc)
			if(dataArray[1]!="text")
			{
				for(var i=0; i<dataArray.length;i++)
				{	
					if(dataArray[i]!="nestedType")
						newElem.setAttribute(dataArray[i],dataArray[i+1]);
					//we have to set the id to allow nesting so 
					else if(dataArray[i]=="id")
					{
						alert("ID is already set to "+newElem.id+" and is not editable");
						continue;
					}
					else
					{
						var p = divElem;
						if (divElem.firstChild.tagName.toLowerCase()=="table" && divElem.firstChild.rows) {
							p = divElem.firstChild.rows[0];
							p = p.insertCell(p.cells.length);
						}
						p.appendChild(newElem);	
						dataArray[i]="annotType";
						var newDataArray=[];
						var index=0;
						for(var j=i;j<dataArray.length; j++)
						{
							newDataArray[index]=dataArray[j];
							index++;
						}

						this.annotate(newDataArray,newElem.id);
						//leaving because we only allow one level of nesting
						//NEED TO ALLOW 2
						return;
					}
					i++;
				}
			}	
			//Add new element to node	
			divElem.appendChild(newElem);	
		
	}

/**
 * @method private TreeItem.prototype.select
 * 	This method is used to highlight the to be selected node or unhighlight the selected code
 * @param boolean toDeselect
 *	If this node is selected or unselected
**/
	TreeItem.prototype.select = function item_select (toDeselect,disableEvent,nativeEvt)
	{
		//this.isSelected = true;

			//Bug fix for highlight section of statestring being incorrect after generateUIString
			//The generateUIString was returning a statestring with the 
			//value of the highlighted object prior to the click rather than the newly 
			//clicked node.Fix was to move these out of if stmt. 
			var oldItem = this.rootItem.selectedItem;
			this.rootItem.selectedItem = this;

			if (!toDeselect)
			{

				if(!disableEvent){
						// For unknown reasons, we have to force the focus rectangle to draw
						if (nativeEvt && nativeEvt.type.toLowerCase()=="click") {
							var f = this.getE('i_a' + this.rootItem.itemID + '_' + this.itemID);
							if (f) {
								try {
									f.focus();
								} catch(e) {};
							}
						}

						if(!this.activeNodeEvent("onHighlight",null,nativeEvt))
						{
							//revert selectedItem to previous node if onHighlight fails
							this.rootItem.selectedItem=oldItem;
							return false;
						}
						if(!this.activeTreeEvent("onNodeHighlight",null,nativeEvt))
						{
							//revert selectedItem to previous node if onNodeHighlight fails
							this.rootItem.selectedItem=oldItem;
							return false;
						}
				}

				//var oldItem = this.rootItem.selectedItem;
				//this.rootItem.selectedItem = this;

				if (oldItem) {
					if(typeof(oldItem)!="string")
						oldItem.select(true);
					//oldItem.isSelected = false;
				}
			}
			
			else
			{
				//Returning this.rootItem.selectedItem to the state it would have been in 
				//if it had skipped the if stmt prior to the bugfix above.
				this.rootItem.selectedItem=oldItem;
				oldItem = null;
			}	
		var o_iicon = document.images['i_img' + this.rootItem.itemID + '_' + this.itemID];
		if (o_iicon) o_iicon.src = this.getImage();

		var labelNormalStyle = this.rootItem.styleClass+"labelNormalStyle";
		var labelHighlightStyle = this.rootItem.styleClass+"labelHighlightStyle";


		if (this.getE('i_txt' + this.rootItem.itemID + '_' + this.itemID) != null)
			//	this.getE('i_txt' + this.rootItem.itemID + '_' + this.itemID).style.fontWeight = toDeselect ? 'normal' : 'bold';
			this.getE('i_txt' + this.rootItem.itemID + '_' + this.itemID).className = toDeselect ? labelNormalStyle : labelHighlightStyle;


/*
		if (this.getE('i_txt' + this.rootItem.itemID + '_' + this.itemID) != null)
		//	this.getE('i_txt' + this.rootItem.itemID + '_' + this.itemID).style.fontWeight = toDeselect ? 'normal' : 'bold';
			this.getE('i_txt' + this.rootItem.itemID + '_' + this.itemID).className = toDeselect ? labelNormalStyle : labelHighlightStyle;

*/
	//	this.upstatus(false,"select");
		window.status = this.itemName + (this.itemTarget ? ' ('+ this.itemTarget + ')' : '');
		return Boolean(this.itemTarget);
	}
/**
 * @method private TreeItem.prototype.check
 * 	This method is used to select the checkbox to select the node or unselect the selected code
 * @param String selectFlag
 *	If this node is selected "ONSELECT" or unselected "ONUNSELECT"
**/
	TreeItem.prototype.check = function item_check(selectFlag, disableEvent, nativeEvt)
	{
		if(selectFlag == "ONSELECT"){

			this.rootItem.checkedItemArray[this.eobj.ID] = this.itemID;
			this.rootItem.checkedEObjectArray[this.eobj.ID] = this.eobj;

		}else{
			this.rootItem.checkedItemArray[this.eobj.ID] = null;
			this.rootItem.checkedEObjectArray[this.eobj.ID] = null;

		}

		this.recursiveCheck(this.rootItem, this, selectFlag);
		
		if(!disableEvent){


			//handler node leve onselect/onunselect event
			 if(!this.activeNodeEvent(selectFlag,null,nativeEvt))
				return false;

			//hanlder tree level onSelect/onunselect event
			if(selectFlag.toUpperCase() == "ONSELECT"){
			 	if(!this.activeTreeEvent("onNodeSelect",null,nativeEvt))
					return false;
			}else{
				if(!this.activeTreeEvent("onNodeUnSelect",null,nativeEvt))
					return false;
			}
		}


		return true;


	}

/**
 * @method private TreeItem.prototype.recursiveCheck
 * 	This method is used to select the children node if the parent node is selected or verse vise
 * @param Object tree
 *	the tree object
 * @param Object selectItem
 *	the parent node
 * @param String selectFlag
 *	If this node is selected "ONSELECT" or unselected "ONUNSELECT"
**/

  	 TreeItem.prototype.recursiveCheck = function(tree, selectItem, selectFlag)
   	 {

		 //select the current selected node
	 	if(selectFlag=="ONSELECT"){

			this.getE('ckbox' + tree.itemID + '_' + selectItem.itemID).checked = true;
		}else{
			this.getE('ckbox' + tree.itemID + '_' + selectItem.itemID).checked = false;
		}


		//select the parent node
		this.backwardRecursive(tree,selectItem.parentItem, selectFlag);

		//select all the children node
		 for(var i=0; i<selectItem.childItems.length; i++)
		 {

			if(selectItem.childItems[i]!=null){

				if (this.getE('ckbox' + tree.itemID + '_' + selectItem.childItems[i].itemID) != null)
					if(selectFlag=="ONSELECT"){
						this.getE('ckbox' + tree.itemID + '_' + selectItem.childItems[i].itemID).checked = true;

						tree.checkedItemArray[selectItem.childItems[i].eobj.ID] = selectItem.childItems[i].itemID;
						tree.checkedEObjectArray[selectItem.childItems[i].eobj.ID] = selectItem.childItems[i].eobj;

					}else{

						this.getE('ckbox' + tree.itemID + '_' + selectItem.childItems[i].itemID).checked = false;
						tree.checkedItemArray[selectItem.childItems[i].eobj.ID] = null;
						tree.checkedEObjectArray[selectItem.childItems[i].eobj.ID] = null;

					}

				this.recursiveCheck(tree, selectItem.childItems[i], selectFlag);
			}

	 	}


	 }

/**
 * @method private TreeItem.prototype.backwardRecursive
 * 	This method is used to select the parent node if all the children node are selected or verse vise
 * @param Object tree
 *	the tree object
 * @param Object selectItem
 *	the parent node
 * @param String selectFlag
 *	If this node is selected "ONSELECT" or unselected "ONUNSELECT"
**/

		 TreeItem.prototype.backwardRecursive = function(tree,parentItem, selectFlag)
		 {


			if(parentItem.eobj!=null){
				if(selectFlag=="ONUNSELECT"){

					for(var j=0; j<parentItem.childItems.length;j++)
					{
						var eobjid = parentItem.childItems[j].eobj.ID;

						if(tree.checkedItemArray[eobjid]!=null)
						{
							return;
						}



					}
					if(this.getE('ckbox' + tree.itemID + '_' + parentItem.itemID)!=null)
						this.getE('ckbox' + tree.itemID + '_' + parentItem.itemID).checked = false;

					tree.checkedItemArray[parentItem.eobj.ID] = null;
					tree.checkedEObjectArray[parentItem.eobj.ID] = null;

				}else{

					for(var j=0; j<parentItem.childItems.length;j++)
					{
						var eobjid = parentItem.childItems[j].eobj.ID;

						if(tree.checkedItemArray[eobjid]==null)
						{
							return;
						}



					}
					if(this.getE('ckbox' + tree.itemID + '_' + parentItem.itemID)!=null)
						this.getE('ckbox' + tree.itemID + '_' + parentItem.itemID).checked = true;
					tree.checkedItemArray[parentItem.eobj.ID] = parentItem.itemID;
					tree.checkedEObjectArray[parentItem.eobj.ID] = parentItem.eobj;


				}
				this.backwardRecursive(tree,parentItem.parentItem, selectFlag);
			}

	 }

/**
 * @method private TreeItem.prototype.getJunctionImageAlt
 * 	This method is used to construct the html print out for that node
**/
	TreeItem.prototype.getJunctionImageAlt = function()
	{
		var childless = !this.hasChildren();
			
		if (this.childItems.length && this.isOpened && !childless)
         return odcTreeimgCollapse; // junction for opened node
		else if (this.childItems.length && !childless )
         return odcTreeimgExpand;  // junction for closed node
		else
		 return "";

	}

/**
 * @method private TreeItem.prototype.getImageAlt
 * 	This method is used to construct the html print out for that node
**/

	TreeItem.prototype.getImageAlt = function()
	{

		if (this.depth >=0)
		{
			var childless = !this.hasChildren();
			// That is unless the node has associated onnodeexpand/collapse or onexpand/collapse hnadlers.
			// In this case the +/- should be present
			// (This is what we do in JWL 2.1, but not clear if it should be continued in JWL 3.0
//			var handlerPresent = (childless&&this.expandCollapseCheck);
			handlerPresent = false;
						
			if (this.childItems.length && this.isOpened && (!childless  || handlerPresent)) {
				if(this.customOpenedImagealt)
					return this.customOpenedImagealt;
				else
					return odcTreeimgFolderOpen.replace(/\{0\}/g, this.itemName);
			} else if (this.childItems.length && (!childless || handlerPresent)) {
				if(this.customClosedImagealt)
					return this.customClosedImagealt;
				else
					return odcTreeimgFolderNormal.replace(/\{0\}/g, this.itemName);
			} else {
				if(this.customOpenedImagealt)
					return this.customOpenedImagealt;
				else if(this.customClosedImagealt)
					return this.customClosedImagealt;
				else
					return odcTreeimgLeaf.replace(/\{0\}/g, this.itemName);
			}

		}




	}

/**
 * @method private TreeItem.prototype.init
 * 	This method is used to construct the html print out for that node
**/
	TreeItem.prototype.init = function item_init()
	{

		var i, img, strSrc, rootId = this.rootItem.itemID, itemId = this.itemID, aOffset = [], currentItem = this.parentItem;

		for (i = this.depth; i > 1; i--) {

			//If its not a workplace tree both parent/child and sibling relationships should be graphically represented
			if(this.rootItem.workPlaceFlag!=true)
				currentItem.isLast()? strSrc = this.rootItem.treeIconObjs["empty"].src : strSrc = this.rootItem.treeIconObjs["line"].src;
			//If it is a workplace tree the only parent/child relationship should be shown.
			else
				//if it is not a child of the root the join may be displayed
				if (i!=2)
					currentItem.isLast()? strSrc = this.rootItem.treeIconObjs["empty"].src : strSrc = this.rootItem.treeIconObjs["line"].src;
				//if it is a child of the root no join should ever be displayed
				else
					strSrc = this.rootItem.treeIconObjs["empty"].src;

			/*aOffset[i] = '<img src="' + strSrc + '" border="0" align="absbottom">';*/
			img = document.createElement("img");
			img.setAttribute("src",strSrc);
			img.setAttribute("alt","");
			img.setAttribute("border","0");
			img.setAttribute("align","absbottom");

			aOffset[i] = img;
			currentItem = currentItem.parentItem;
						
		}
		
		var item = this.rootItem.selectedItem;
		if(item) {
			if(typeof(item) == "string") {
				if(item==this.eobj.getSignature())
					this.rootItem.selectedItem = this;
			}
		}

		var returnArr=[], div, t, a, a2, td, tr, inp, c, newW;

		if(!this.rootItem.rootVisibleFlag&&this.itemID==0) {
			//root is not visible
			if(this.childItems.length) {
				div = document.createElement("div");
				div.style.borderWidth ="0px";
				div.setAttribute("id","i_div" + this.rootItem.itemID + "_" + this.itemID);
				div.style.display = "none";				
			}
			returnArr[0] = div;
			return returnArr;

		} else {
			
			//Style classes
			var l, nodeStyle = this.rootItem.styleClass+"nodeStyle";
				
			//Create table element for node
			t = document.createElement("table");
			t.setAttribute("id","i_tab" + this.rootItem.itemID + "_" + this.itemID);
			t.cellPadding = t.border = t.cellSpacing = "0";
			t.className=nodeStyle;
			tr = t.insertRow(this.currentRowNumber++); //document.createElement("tr");
			tr.setAttribute("id","i_tr" + this.rootItem.itemID + "_" + this.itemID);

			//Add tabular data for the node (if this is a tree table)
			if(this.rootItem.treeTableFlag)
				tr=this.addTabularData(tr);

			// Create the cell element for the node
			td=this.createNodeCell(tr);
			// Calc the width of this element (if one is given)
			newW = this.rootItem.getColWidth(0);
			if (newW!=null&&newW.W!=null) {
				// Set the width on BOTH the td and the label inside (so the contents of the cell will wrap)
				td.width=newW.W;
				td.style.width=newW.WS;
			}

			// In order to wrap the "label" in the tree node, we need
			// to set up some nested one row high tables so that the label
			// can be in a cell of it's own (so it can either wrap or truncate
			// along a left margin)
			var lineTR, lineTD=null, lineT = document.createElement("table");
			lineT.cellPadding = lineT.border = lineT.cellSpacing = "0";
			lineTR = lineT.insertRow(0);
			// Note the following is provisional -- we need the table somewhere where it has a size that can be measured	
			lineT.style.display="none";
			document.body.insertBefore(lineT, document.body.firstChild);

			if (this.depth > 0) {		
				if (lineTD==null) lineTD = lineTR.insertCell(0);
				for(l=0; l<aOffset.length; l++) {
					if(aOffset[l] != null) lineTD.appendChild(aOffset[l]);
				}
				//Create nodes junction image and if required, its associated events 
				a=this.getJunctionLnk();
			}
			
			// If the select option is available for the nodes add the check box and related event handler
			if(odcTrees[this.rootItem.itemID].enableSelect) {
				inp = document.createElement("input");
				inp.setAttribute("type","checkbox");	
				inp.setAttribute("id","ckbox" + this.rootItem.itemID + "_" + this.itemID);	
				inp.clickMethod=  function(e){
					e = (e!=null)?e:event;
					odcTrees[ rootId ].check(document.getElementById("ckbox" +rootId+ "_" +itemId), itemId, e ); 
				}
				addStandardEvent(inp, "click", inp.clickMethod, true);		
			}
				
			// Create Node image element and if required, its related event handlers .		
			a2 = this.getImageLnk();
			
			// If it is not the root node add the nodes junction element
			if(this.depth > 0) {
				if (lineTD==null) lineTD = lineTR.insertCell(0);
				lineTD.appendChild(a);
			}

			// Add checkbox element if select is enabled
			if(odcTrees[this.rootItem.itemID].enableSelect) {
				if (lineTD==null) lineTD = lineTR.insertCell(0);
				lineTD.appendChild(inp);
			}

			// We now have everything "left" of the image/label element (all the junction lines, selection, ...)
			// Measure the size of that cell and force it to stay that size
			lineT.style.display="";
			if (lineTD!=null) {
				var tw = lineTD.offsetWidth;
				lineTD.width = tw;
				lineTD.style.width = tw+"px";
			}

			// Now append another cell (if needed) that contains the image/label in this row
			td.appendChild(lineT);
			lineTD = lineTR.insertCell(lineTR.cells.length);
			lineTD.appendChild(a2);

			// Div added to hold children if they exist
			if(this.childItems.length > 0) {
				c = document.createElement("div");
				c.style.borderWidth = "0px";				
				c.id ="i_div" + this.rootItem.itemID + "_" + this.itemID;
				returnArr[1] = c;
			}

			returnArr[0] = t;
			return returnArr;

		}
	}

	TreeItem.prototype.createNodeCell=function(tr)
	{
	
		//Different style used for tree node td if it is a tree table
		var tdStyle = (!this.rootItem.treeTableFlag)?(this.rootItem.styleClass+"tdStyle"):(this.rootItem.styleClass+"tdTreeTabStyle");
			
		var td = tr.insertCell(this.currentCellNumber++);
		td.setAttribute("id","i_td" + this.rootItem.itemID + "_" + this.itemID);
		td.className=tdStyle;
							
		return td;
	}
	
	//Get Junction Lnk gets Dom element containin appropriate DOM element for the junction of the node.
	//Dependent on state of ShowSystemIcon flag, existance of children, existance of custom event handlers and if root node iksis visible]
	TreeItem.prototype.getJunctionLnk=function()
	{
		var rootId = this.rootItem.itemID, itemId = this.itemID, a, aImg, iconStyle = this.rootItem.styleClass+"nodeImageStyle", linkStyle = this.rootItem.styleClass+"linkTextStyle";
	
		//Scenario showSystemIcon is set to false. No standard events should be attached to this node
		if(this.showSystemIconFlag=="false")
		{
			a = document.createElement("img");
			a.setAttribute("src",this.getIndentImage());	
			a.setAttribute("alt",""); // alt is required for xhtml (2.0), so we add as empty string
			a.className = iconStyle;
			a.setAttribute("border","0");
			a.setAttribute("align","absbottom");
		}
		//Or if the node has no children and no asscociated expand or collapse handler then image should only be a join 
		//and there should be no standard events attached
		else if(!this.hasChildren())
		{
			a = document.createElement("img");
			a.setAttribute("src",this.getJunctionImage());	
			a.setAttribute("alt",""); // alt is required for xhtml (2.0), so we add as empty string
			a.className = iconStyle;
			a.setAttribute("border","0");
			a.setAttribute("align","absbottom");
		}
		//Or else if the node has children and is set to show the system icons then plus/minus icons should be added along
		//with the standard events for an active tree node.
		else				
		{					
			a = document.createElement("a");
			a.setAttribute("href","javascript:;");	
			a.className = linkStyle;
			a.setAttribute("id","j_a" + this.rootItem.itemID + "_" + this.itemID);
					
			try
			{
				//Add pointers to methods to element object
				a.clickMethod=function (e) {
					e = (e!=null)?e:event;
					if(typeof(odcTrees)!="undefined")
						return odcTrees[ rootId ].toggle( itemId,e );
					else
						return null; 
				}
						
				a.moverMethod=function () {
					if(typeof(odcTrees)!="undefined")
						return odcTrees[ rootId ].mover( itemId );
					else
						return null; 
				}
				
				a.moutMethod=function () {
					if(typeof(odcTrees)!="undefined")
						return odcTrees[ rootId ].mout( itemId );
					else
						return null; 
				}
						
				//Add dom element to nodeEvents array
				addStandardEvent(a, "click", a.clickMethod, true);
				addStandardEvent(a, "mouseover",  a.moverMethod, true);
				addStandardEvent(a, "mouseout", a.moutMethod, true);
			}
			catch(e)
			{
				alert(e);
			}
				
			aImg = document.createElement("img");
			aImg.setAttribute("src",this.getJunctionImage());	
			aImg.setAttribute("alt",this.getJunctionImageAlt());
			//Title attribute provides cross browser tooltips, alt is IE specific.
			aImg.setAttribute("title",this.getJunctionImageAlt());
			aImg.setAttribute("border","0");
			aImg.setAttribute("align","absbottom");
			//aImg.setAttribute("name","j_img" + this.rootItem.itemID + "_" + this.itemID);				
			aImg.setAttribute("id","j_img" + this.rootItem.itemID + "_" + this.itemID);				
			aImg.className = iconStyle;				
			a.appendChild(aImg);	
		
		}
		return a;				
	}
	
	TreeItem.prototype.getImageLnk = function ()
	{
		var rootId = this.rootItem.itemID, itemId = this.itemID;
		var a, aImg, lbl, t, tr, td1, td2, iconStyle = this.rootItem.styleClass+"nodeImageStyle", linkStyle = this.rootItem.styleClass+"linkTextStyle", nodeLableStyle = this.rootItem.styleClass+"labelNormalStyle";

		a=document.createElement("a");
		a.setAttribute("href","javascript:;");	
		a.className = linkStyle;
		a.id = "i_a" + this.rootItem.itemID + "_" + this.itemID;
		
		//Only add expand/collapse functionality if the item has children or if item has no children but has
		//attached expand/collapse event listener
		if(this.hasChildren())
		{
			a.dblClickMethod=function (e) {
				e = (e!=null)?e:event;
				return odcTrees[ rootId ].toggle( itemId,e ); 
			}
			addStandardEvent(a, "dblclick", a.dblClickMethod, true);
		}

		//Method pointers needed to detach the method on page unload
		a.moverMethod=function () {
			if(typeof(odcTrees)!="undefined")
				return odcTrees[ rootId ].mover( itemId );
			else
				return null;
		}
			
		a.moutMethod=function () {
			if(typeof(odcTrees)!="undefined")
				return odcTrees[ rootId ].mout( itemId );
			else
				return null;
		}
		a.clickMethod=function (e) {
			e = (e!=null)?e:event;
			if(typeof(odcTrees)!="undefined")
				return odcTrees[ rootId ].select( itemId,e );
			else
				return null;
		}

		addStandardEvent(a, "click", a.clickMethod, true);
		addStandardEvent(a, "mouseover", a.moverMethod , true);	
		addStandardEvent(a, "mouseout", a.moutMethod, true);

		//Add drag&drop capabilities to node if: 
		//1. Tree is enabled to have drag&drop capabailities by the user at tree level using the 
		//   onnodedrop tag attribute 
		//2. Node is NOT a terminal (leaf) node
		if(this.rootItem.enableDrop&&this.childItems.length>0)
			a=this.addDropEvents(a);
		//end if drag&drop enabled for tree

		// To get the text next to the icon to "wrap" correctly, we have to put the label in a little table
		t = document.createElement("table");
		t.cellPadding = t.border = t.cellSpacing = "0";
		tr = t.insertRow(0);
		td1 = tr.insertCell(0);
		td2 = tr.insertCell(1);

		aImg = document.createElement("img");
		aImg.setAttribute("src",this.getImage());	
		aImg.setAttribute("alt",this.getImageAlt());
		//Title attribute provides cross browser tooltips, alt is IE specific.
		aImg.setAttribute("title",this.getImageAlt());
		aImg.setAttribute("border","0");
		aImg.setAttribute("align","absbottom");	// fix? why force this and not do it in CSS?
		aImg.setAttribute("id","i_img" + this.rootItem.itemID + "_" + this.itemID);
		aImg.className = iconStyle;

		lbl = document.createElement("label");
		lbl.setAttribute("id","i_txt"+ this.rootItem.itemID + "_" + this.itemID);
		lbl.className = nodeLableStyle;

		// The following will turn on/off wrapping
		// The user can control it via CSS (set it on the label CSS class)
//		lbl.style.whiteSpace = "nowrap";
//		lbl.style.overflow="hidden";
		lbl.style.display="block";
		lbl.style.width="100%";
		// The following should be 'enable-able' at some point (via a new attribute)
//		lbl.title=this.itemName;
		lbl.appendChild(document.createTextNode(this.itemName));

		td1.appendChild(aImg);
		td2.appendChild(lbl);
		a.appendChild(t);

		return a;

	}

/**
 * @method private TreeItem.prototype.addTabularData
 * 	Adds element as extra data to the tree table
**/

	TreeItem.prototype.addTabularData  = function (trElem)
	{	
			//Style classes
			var tdTableStyle = this.rootItem.styleClass+"tdTableStyle", nodeLableStyle = this.rootItem.styleClass+"labelNormalStyle";
			
			//eobj contains all objects details
			var eobjMembers=this.eobj.Members;
			
			//A variable in the datasource that contains the string of column data passed in via the treeColumnData tag
			var dataStr=this.rootItem.Adapter.dataHandler[this.eobj.EClass.Name];
			var k, b, val, len, refArr=[], dataArr=[];
			
			if(dataStr!=null) {
				refArr=dataStr.split(";");
				for(b=0;b<refArr.length;b++) {
					val=null; len=null;
					if(refArr[b]!="" && refArr[b] != null && refArr[b] != "null")
						val=this.eobj.eGet(refArr[b]);
					//Code to cater for if data is provided from an array in a bean rather 
					//than individual properties(Request from AIMUL team)
					if(this.eobj.childrenlen) {
						if (refArr[b]!=""  && refArr[b] != null && refArr[b] != "null")
							len=(this.eobj.childrenlen(refArr[b]));
					}
					dataArr[b]=val;
				
					if(len!=null) {
						if(len>b) dataArr[b]=val[b];
					}
				}
			}
			
			//number of column to be appended to the already existing tree control column is total column headers -1
			var lbl, td, txt, newW, numCols=this.rootItem.treeTableArr.length-1;

			//For each requested table column value
			for(k=numCols-1;k>=0;k--)
			{
				//create element with this info and add to array
				td = trElem.insertCell(this.currentCellNumber++);//document.createElement("td");
				td.setAttribute("id","i_td_"+this.currentCellNumber+"_" + this.rootItem.itemID + "_" + this.itemID);
				td.className=tdTableStyle;

				lbl = document.createElement("label");
				lbl.setAttribute("id","i_txt_"+this.currentCellNumber+"_"+this.rootItem.itemID + "_" + this.itemID);
				lbl.className = nodeLableStyle;

				newW = this.rootItem.getColWidth(k+1);
				if (newW!=null&&newW.W!=null) {
					// Set the width on BOTH the td and the label inside (so the contents of the cell will wrap)
					td.width=newW.W;
					td.style.width=newW.WS;
					lbl.style.display="block";
					lbl.style.width=newW.WS;
				}
				// Add in the text
				if(dataArr!=null)
					txt = (dataArr[k]!=null)?(document.createTextNode(" "+dataArr[k])):(document.createTextNode(""));
				//Blank text element
				else
					txt = document.createTextNode("");						
				lbl.appendChild(txt);
				td.appendChild(lbl);
			}
			
		return trElem;
	}

/**
 * @method private TreeItem.prototype.addDropEvents
 * 	This method is used add all event and style handling for drop events cross browser
 * @param DOMElement element
 *	Element that events are to be attached to
**/
	TreeItem.prototype.addDropEvents = function(element)
	{
		//local id variables
		var rootId=this.rootItem.itemID, itemId=this.itemID;
		
		//Method pointers needed to detach the method on page unload
		element.dragEnterMethod=function(e){odcTrees[ rootId ].dover( itemId,e );}
		element.dragOverMethod=function(e){odcTrees[ rootId ].dover( itemId,e );}
							
		//Add onvalidatedragdrop handlers for dragover and dragenter
		//Styles for entering a drop zone are also set by this event handlers. 
		addStandardEvent(element, "dragenter", element.dragEnterMethod , true);		
		addStandardEvent(element, "dragover", element.dragOverMethod ,true );
		
		//Event Listeners set here to ensure styles are returned to normal after leaving the drop area.
		//Can't use generic addStandardEvent here because Mozilla event is dragexit and IE is ondragleave
		if(isIE())
		{
			//Set pointer to event method
			element.dragLeaveMethod=function(){odcTrees[ rootId ].dout(itemId);}
							
			//Extra attributes added to the DOM Element to keep track of events added
			addStandardEvent(element,"dragleave",element.dragLeaveMethod,true);
			
		}
		else
		{
			element.addEventListener("dragexit"
										,function(e)
										{
											odcTrees[ rootId ].dout(itemId); 
										}
										,false );
		}
					 	
		//Adding drop event listener and corresonding handler
		//Can't use generic addStandardEvent here because Mozilla event is dragdrop and IE is ondrop
		if(isIE())
		{
			//Set pointer to event method
			element.dropMethod=function(){odcTrees[ rootId ].drop(itemId, window.event); }
			addStandardEvent(element,"drop",element.dropMethod,true);
		}
		else
		{
			element.addEventListener("dragdrop"
										,function(e)
										{
											odcTrees[ rootId ].drop(itemId, e); 
											//Required to stop loading of file dropped onto page
											e.cancelBubble = true; 
											if (e.stopPropagation) 
												e.stopPropagation();
										}
										,false );
		}
	
		return element;				
	}

/**
 * @method private TreeItem.prototype.upstatus
 * 	This method is used to update the brower's status bar
 * @param boolean b_clear
 *	If true, clear all the statement, otherwise, print out the node name etc.
**/
	TreeItem.prototype.upstatus = function item_upstatus (b_clear,type)
	{

			window.status = b_clear ? '' : this.itemName + (this.itemTarget ? ' ('+ this.itemTarget + ')' : '');

			var labelMouseoverStyle = this.rootItem.styleClass+"labelMouseoverStyle";
			var labelNormalStyle = this.rootItem.styleClass+"labelNormalStyle";
			var labelDragoverStyle = this.rootItem.styleClass+"labelDragoverStyle";
			var labelHighlightStyle = this.rootItem.styleClass+"labelHighlightStyle";
			if (this.getE('i_txt' + this.rootItem.itemID + '_' + this.itemID) != null)
			{
				if(type=="mouseover")
				{

					this.getE('i_txt' + this.rootItem.itemID + '_' + this.itemID).className = labelMouseoverStyle;

				}
				else if(type=="mouseout")
				{
					var item = odcTrees[this.rootItem.itemID].selectedItem;

					if(item&&typeof(item)!="string"&&item.eobj.ID == this.eobj.ID)
					{
						//restore the highlighted node's style
						this.getE('i_txt' + this.rootItem.itemID + '_' + this.itemID).className = labelHighlightStyle;

					}
					else
						this.getE('i_txt' + this.rootItem.itemID + '_' + this.itemID).className = labelNormalStyle;
				}
				else if(type=="dragover")
				{
						this.getE('i_txt' + this.rootItem.itemID + '_' + this.itemID).className = labelDragoverStyle;
				}
				else if(type=="dragout"||type=="dropout")
				{
					var item = odcTrees[this.rootItem.itemID].selectedItem;

					//if already highlighted retain highlightStyle
					if(item&&typeof(item)!="string"&&item.eobj.ID == this.eobj.ID)
					{
						//restore the highlighted node's style
						this.getE('i_txt' + this.rootItem.itemID + '_' + this.itemID).className = labelHighlightStyle;
					}
					else
						this.getE('i_txt' + this.rootItem.itemID + '_' + this.itemID).className = labelNormalStyle;
				}
				else
				{
					this.getE('i_txt' + this.rootItem.itemID + '_' + this.itemID).className = labelNormalStyle;
				}



			}


	}


/**
 * @method private TreeItem.prototype.isLast
 * 	This method is used to determine if this node is leaf
**/
	TreeItem.prototype.isLast = function()
	{
		return this.itemIndex == this.parentItem.childItems.length - 1;
	};

/**
 * @method public getAnchorElem
 * Returns the HTML anchor element reference for the current item. 
 **/
	TreeItem.prototype.getAnchorElem = function() 
	{
		var id = "i_a"+this.rootItem.itemID+"_"+this.itemID;
		return document.getElementById(id);
	}
	
/**
 * @method public getParent
 * Returns the parent TreeItem. 
 **/
	TreeItem.prototype.getParent = function() 
	{
		return this.parentItem;
	}

if(!odcTrees) 
{
	var odcTrees = [];  // global pointer to multiple odcTrees
}
