


/**
 * @class public XML2EMF
 * @constructor XML2EMF
 * 	 This class translates between EMF and XML calls
 *	 This will allow the ODC components to use XML Data. 
 * @param XMLData xmlData
 *	 The XMLData object 
 **/


function XML2EMF(xmlData, parent)
{
	this.xmlData = xmlData;

	this.ID = NewID();
	
	this.parent = parent;
	
	this.childrenref = new Array();

	this.isXML2EMF = true;
	
	// todo set this properly, this is not the correct way to get name.
	this.EClass = this.xmlData;
	this.EClass.Name = this.xmlData.name();
	
	
	this.Members = new Array();

	this.PropertyBinders = IndexMultiMapCreate(); // public
	this.PropertyBinders.Add = function(propertyBinder)	// public
	{


		IndexMultiMapAdd(this, propertyBinder.PropertyName, propertyBinder);



		return propertyBinder;
	}
	this.PropertyBinders.Get = function(name)	// public
	{


		var objArray = IndexMultiMapGet(this, name);



		return objArray;
	}
	this.PropertyBinders.Remove = function(propertyBinder)
	{


		var res = IndexMultiMapRemove(this, propertyBinder);



		return res;
	}

	this.PropertyArrayBinders = IndexMultiMapCreate(); // public
	this.PropertyArrayBinders.Add = function(propertyArrayBinder)	// public
	{


		IndexMultiMapAdd(this, propertyArrayBinder.PropertyName, propertyArrayBinder);



		return propertyArrayBinder;
	}
	this.PropertyArrayBinders.Get = function(name)	// public
	{


		var objArray = IndexMultiMapGet(this, name);



		return objArray;
	}
	this.PropertyArrayBinders.Remove = function(propertyArrayBinder)
	{


		var res = IndexMultiMapRemove(this, propertyArrayBinder);



		return res;
	}






}


/**
 * @method public XML2EMF.prototype.Refresh
 * 	This method refreshes all propertybinders
 * @param String name
 *	Name of a data member.
 * @param Object value
 *	Value of a data member 
 **/
XML2EMF.prototype.Refresh = function(name, value)
 {
  
    var propertyBinders = this.PropertyBinders;
	var propertyNamedBinders = propertyBinders.Get(name)
	if ( propertyNamedBinders != null )
	{
		for (var i = propertyNamedBinders.length - 1; i >= 0; --i)
		{
			var propertyBinder = propertyNamedBinders[i];
			propertyBinder.FireValueChanged(name, value, propertyBinder.index);
		}
	}
}

/**
 * @method public XML2EMF.prototype.addChildrenRef
 * 	This adds an XML2EMF object to the array of children objects 
 *  for this object. 
 * @param String name
 *	Name of the Reference.
 * @param XML2EMF ref
 *	The reference to the child object.
 * @param Int i
 *	The index in the array to add the object
 **/
XML2EMF.prototype.addChildrenRef = function(name, ref, i)
{
	if (i != null) {
		this.childrenref[name+i] = ref;
	}
	else {
		this.childrenref[name] = ref;
	}
}

/**
 * @method public XML2EMF.prototype.getChildrenRef
 * 	This gets a child object given the name and index number.
 * @param String name
 *	Name of the Reference to get.
 * @param Int i
 *	The index of the object to get.
 * @return XML2EMF ref
 *	The reference to the child object returned.
 **/
XML2EMF.prototype.getChildrenRef = function(name, i)
{
	if (i != null)
	{
		try {
			var ref = this.childrenref[name+i];
		}
		catch (e)
		{
			var ref = null;
		}
	} 
	else {
		try {
			var ref = this.childrenref[name];
		}
		catch (e)
		{
			var ref = null;
		}
	}

	return ref;
}



/**
 * @method public XML2EMF.prototype.eGet
 * 	This gets the value from the XMLData object.
 * @param String name
 *	Name of a data member.
 * @return Object value
 *	Value of a data member 
 **/
XML2EMF.prototype.eGet = function(name)
{
	if (name == null)
	{
		return null;
	}

	var child  = this.xmlData.child(name);

	if ( child == null || (child.length() == 0) )
	{
		return null;
	}

	//var tmp1 = child.elementByIndex(0).children();

	// todo use proper check to see if it is text only in this node
	if ( child.elementByIndex(0).hasSimpleContent() )
//	if ( name == "name" )
	{
	//todo set this by proper index to allow binders to change it.
		var val1  = child.elementByIndex(0);
		var value = val1.text().toString();
	//	var value = child.elementByIndex(0).text();

		var test1 = isNaN(value);
		var test2 = (typeof(value) == "string" );		
		var test4 = true;
		if(test2)
			if(value.length == 0)
				test4 = false;
		
		// if is a number
		if(test4)
			if (!test1) 			
				value = parseFloat(value);
		

	}
	else
	{
	  //if here, complicated.
		var value = new Array();
		for (var i=0; i<child.length(); i++)
		{
			if (this.getChildrenRef(name, i) != null)
			{
				value[i] = this.getChildrenRef(name, i);
			}
			else {
				value[i] = new XML2EMF(child.elementByIndex(i));
				this.addChildrenRef(name, value[i], i);
			}
			
		}	
				
	}
			
	


	return value;
}



/**
 * @method public XML2EMF.prototype.eSet
 * 	This sets the value of the XMLData object.
 * @param String name
 *	Name of a data member.
  * @param String value
 *	Name of a data member.
 * @return boolean
 *	true for sucess.
 **/
XML2EMF.prototype.eSet = function(name, value)
{
	if (this.GetMember(name) != null)
	{
		//***ADAM: this is better:
		//this.xmlData.child(name).elementByIndex(0).put("*",value);
		this.xmlData.child(name).elementByIndex(0).xmlNode.childNodes[0].nodeValue = value;
		
	//	var tmp1 = this.xmlData.child(name).elementByIndex(0);
			
	}
	else {
		
		var newattr = new hX_5.XMLData("<"+name+">"+value+"</"+name+">");
		var newXML2EMF = new XML2EMF(newattr);
		
		this.eAdd(name,newXML2EMF);		
	
	}
	
	this.Refresh(name, value);
	// todo check if set is sucessful
	
	return true;
}

/**
 * @method public XML2EMF.prototype.CreateEObject
 * 	This creates an XML2EMF object with a root xml node.
 * @param String propertyName
 *	The name of the node.
 * @return XML2EMF
 *	The XML2EMF object created.
 **/
XML2EMF.prototype.CreateEObject = function(propertyName)
{

	// In the original EMF this function created a blank EObject
	// based on the EClass which contained all the information
	// on the structure needed to create this.
	// This is not available with the xmldata so here a xmlData
	// object with one node will be created and extra nodes for 
	// the values will be added the eSet function.

	var newObj = new hX_5.XMLData("<"+propertyName+"> </"+propertyName+">");
	var eXmlObj = new XML2EMF(newObj); 
	
	
	var existingProperties = this.eGet(propertyName);
	var members = existingProperties[0].Members;
	
	var i;
	
	
	// This will only work if an existing similar object already exists
	for (i=0; i< members.length; i++)
	{
		eXmlObj.Members[i] = new Object(); 
			
		eXmlObj.Members[i].Name = members[i].Name;
		eXmlObj.Members[i].EStructuralFeature = new Object();
		eXmlObj.Members[i].EStructuralFeature.CLASSTYPE = members[i].EStructuralFeature.CLASSTYPE
		eXmlObj.Members[i].Value = null;
		
	}	
	
	
	
	return eXmlObj;

}

/**
 * @method public XML2EMF.prototype.eAdd
 * 	This adds a child XML2EMF object to this object.
 * @param String name
 *	The name of the node.
 * @param value
 *	The XML2EMF object to be added.
 **/
XML2EMF.prototype.eAdd = function(name, value)
{
	var tmpList = this.xmlData.child(name);
	
	if (tmpList !=null )
	{
		var length = tmpList.length();
	}
	else {
		var length = 0;
	}
	
	this.addChildrenRef(name,value,length);
	
	this.xmlData.appendChild(value.xmlData);
	this.Refresh(name,this);	

}


/**
 * @method public XML2EMF.prototype.eRemove
 * 	This removes a child XML2EMF object from this object.
 * @param String name
 *	The name of the node.
 * @param value
 *	The XML2EMF object to be removed.
 **/
 
 XML2EMF.prototype.eRemove = function(name, value, isDelete, refreshRequired)
 {
 		var i = 0;
 		
 		var childelements = this.xmlData.child(name);
 		var elementRemoved = false;
 		while(this.childrenref[name+i] != null)
 		{
 			// remove the element from the xml data list
 			if (value.ID == this.childrenref[name+i].ID)
 			{
 				childelements.remove(i);				
 				elementRemoved = true;
 			}	
 			// reorganise the array to reflect this
 			if ( elementRemoved )
 			{
 					this.childrenref[name+i] = this.childrenref[name+i+1];					
 			} 			
 			i++;	
 		}
 /*		for (i = 0; i < this.PropertyArrayBinders.length; ++i)
		{
			this.PropertyArrayBinders[i].FireRowDeleted(name,value);
		} */
		
		this.Refresh(name,this);
 
 
 }


/**
 * @method public XML2EMF.prototype.getSignature
 * 	This returns a signature for this object.
 * @return String
 *	The signature
 **/
XML2EMF.prototype.getSignature = function()
{
	// todo generate proper signature
	return this.EClass.Name+this.ID;
}


/**
 * @method public XML2EMF.prototype.GetMember
 * 	This gets the member given a name
 * @param String name
 *	The name of the member.
 * @return XML2EMF
 *	The member.
 **/
XML2EMF.prototype.GetMember = function(name)
{

	//todo - this is wrong, need to check for members in xmldata somehow.
	var res = new Object();
	
	
	
	res.EStructuralFeature = new Object();
	
	
	var member = this.eGet(name);
	if (member != null)
	{
		var type = typeof(member);
		
		if (typeof(member) == "object")
		{
		//	res.Name = member[0].EClass.Name;
			res.Name = this.EClass.Name;
			res.EStructuralFeature.CLASSTYPE = 0x4;
		}
		else {
		//	res.Name = this.EClass.Name;
			res.Name = name;
			res.EStructuralFeature.CLASSTYPE = 0x1;
		}
		res.Value = member;
	}
	else {
		res = null;
	}
	
	
	// This is where the Members array of this object is created
	// If the the member hasn't been seen before, it is added to 
	// the array.
	var i;
	if ( !name )
	{
		var addMember = false;
	}
	else {
		var addMember = true;
	}
	for (i=0; i< this.Members.length; i++)
	{
		
		if ( this.Members[i].Name == name )
		{
			addMember = false;
			break;
		}
	}	
	if ( addMember && (res!=null) )
	{
		this.Members[this.Members.length] = res;
	}
	return res;
}


/**
 * @method public XML2EMF.prototype.FindMember
 * 	This finds the member given a name
 * @param String name
 *	The name of the member.
 * @return XML2EMF
 *	The member.
 **/
XML2EMF.prototype.FindMember = function(name)
{
/*	try {
		var child  = this.xmlData.child(name);
	}
	catch (e)
	{
		return null;
	}
	if (child != null)
	{
		var member = new Object();
		member.Value = new Array();
		for (var i=0; i<child.length(); i++)
		{
			
				member.Value[i] = new XML2EMF(child.elementByIndex(i),this);
			
		}	
		return member;
	}
*/

	var member = this.GetMember(name);
	return member;

}

/**
 * @method public XML2EMF.prototype.toStr
 * 	This gets a string of the xmldata contained in this object
 * @return String
 *	a string representation of the xmldata this object
 *  refers to.
 **/
XML2EMF.prototype.toStr = function()
{
	return this.xmlData.toString();

}

